@@ -16,8 +16,9 @@ import logging
 
 from oslo_config import cfg
 
-from neutron.i18n import _LW
+from vmware_nsx._i18n import _, _LW
 from vmware_nsx.common import exceptions as nsx_exc
+from vmware_nsx.extensions import routersize
 
 LOG = logging.getLogger(__name__)
 
@@ -73,6 +74,7 @@ base_opts = [
                deprecated_group='NVP',
                help=_("The mode used to implement DHCP/metadata services.")),
     cfg.StrOpt('replication_mode', default=ReplicationModes.SERVICE,
+               choices=(ReplicationModes.SERVICE, ReplicationModes.SOURCE),
                help=_("The default option leverages service nodes to perform"
                       " packet replication though one could set to this to "
                       "'source' to perform replication locally. This is useful"
@@ -153,26 +155,35 @@ cluster_opts = [
                       "be used by logical services like dhcp and metadata")),
     cfg.StrOpt('nsx_default_interface_name', default='breth0',
                deprecated_name='default_interface_name',
-               help=_("Name of the interface on a L2 Gateway transport node"
+               help=_("Name of the interface on a L2 Gateway transport node "
                       "which should be used by default when setting up a "
                       "network connection")),
 ]
 
 nsx_common_opts = [
     cfg.StrOpt('nsx_l2gw_driver',
-               help=_("Class path for the L2 gateway backend driver"))
+               help=_("Class path for the L2 gateway backend driver")),
+    cfg.StrOpt('locking_coordinator_url',
+               deprecated_group='nsxv',
+               help=_('A URL to a locking mechanism coordinator')),
 ]
 
 nsx_v3_opts = [
-    cfg.StrOpt('nsx_user',
+    cfg.StrOpt('nsx_api_user',
+               deprecated_name='nsx_user',
                default='admin',
                help=_('User name for the NSX manager')),
-    cfg.StrOpt('nsx_password',
+    cfg.StrOpt('nsx_api_password',
+               deprecated_name='nsx_password',
                default='default',
                secret=True,
                help=_('Password for the NSX manager')),
-    cfg.StrOpt('nsx_manager',
-               help=_('IP address of the NSX manager')),
+    cfg.ListOpt('nsx_api_managers',
+                deprecated_name='nsx_manager',
+                help=_('IP address of one or more NSX managers separated '
+                       'by commas. The IP address can optionally specify a '
+                       'scheme (e.g. http or https) and port using the format '
+                       '<scheme>://<ip_address>:<port>')),
     cfg.StrOpt('default_overlay_tz_uuid',
                deprecated_name='default_tz_uuid',
                help=_("This is the UUID of the default NSX overlay transport "
@@ -192,24 +203,60 @@ nsx_v3_opts = [
                       "gateway service plugin.")),
     cfg.IntOpt('retries',
                default=10,
-               help=_('Maximum number of times to retry API request')),
+               help=_('Maximum number of times to retry API requests upon '
+                      'stale revision errors.')),
     cfg.StrOpt('ca_file',
                help=_('Specify a CA bundle file to use in verifying the NSX '
-                      'Manager server certificate.')),
+                      'Manager server certificate. This option is ignored if '
+                      '"insecure" is set to True. If "insecure" is set to '
+                      'False and ca_file is unset, the system root CAs will '
+                      'be used to verify the server certificate.')),
     cfg.BoolOpt('insecure',
                 default=True,
                 help=_('If true, the NSX Manager server certificate is not '
-                       'verified. If false, then the default CA truststore is '
-                       'used for verification. This option is ignored if '
-                       '"ca_file" is set.')),
+                       'verified. If false the CA bundle specified via '
+                       '"ca_file" will be used or if unsest the default '
+                       'system root CAs will be used.')),
+    cfg.IntOpt('http_timeout',
+               default=10,
+               help=_('Time before aborting a HTTP connection to a '
+                      'NSX manager.')),
+    cfg.IntOpt('http_read_timeout',
+               default=180,
+               help=_('The time in seconds before aborting a HTTP read '
+                      'response from a NSX manager.')),
+    cfg.IntOpt('http_retries',
+               default=3,
+               help=_('Maximum number of times to retry a HTTP connection.')),
+    cfg.IntOpt('concurrent_connections', default=10,
+               help=_("Maximum concurrent connections to each NSX "
+                      "manager.")),
+    cfg.IntOpt('conn_idle_timeout',
+               default=10,
+               help=_('Ensure connectivity to the NSX manager if a connection '
+                      'is not used within timeout seconds.')),
+    cfg.IntOpt('redirects',
+               default=2,
+               help=_('Number of times a HTTP redirect should be followed.')),
     cfg.StrOpt('default_tier0_router_uuid',
                help=_("Default tier0 router identifier")),
-    cfg.StrOpt('default_switching_profile_dhcp_uuid',
-               help=_("UUID of the default NSX switching profile to allow "
-                      "DHCP traffic. This will be applied on the DHCP ports, "
-                      "if neutron's DHCP service is enabled. This profile "
-                      "must be created on the backend, out of band, with DHCP "
-                      "Server/Client Block disabled.")),
+    cfg.IntOpt('number_of_nested_groups',
+               default=8,
+               help=_("The number of nested NSGroups to use.")),
+    cfg.StrOpt('metadata_mode',
+               default=MetadataModes.DIRECT,
+               help=_("If set to access_network this enables a dedicated "
+                      "connection to the metadata proxy for metadata server "
+                      "access via Neutron router. If set to dhcp_host_route "
+                      "this enables host route injection via the dhcp agent. "
+                      "This option is only useful if running on a host that "
+                      "does not support namespaces otherwise access_network "
+                      "should be used.")),
+    cfg.BoolOpt('metadata_on_demand',
+                default=False,
+                help=_("If true, an internal metadata network will be created "
+                       "for a router only when the router is attached to a "
+                       "DHCP-disabled subnet.")),
 ]
 
 DEFAULT_STATUS_CHECK_INTERVAL = 2000
@@ -285,7 +332,7 @@ nsxv_opts = [
                        'edge_size: compact, large, xlarge, quadlarge '
                        'and default is large.')),
     cfg.IntOpt('retries',
-               default=10,
+               default=20,
                help=_('Maximum number of API retries on endpoint.')),
     cfg.StrOpt('mgt_net_moid',
                help=_('Network ID for management network connectivity')),
@@ -298,13 +345,21 @@ nsxv_opts = [
                       'metadata proxy')),
     cfg.ListOpt('nova_metadata_ips',
                 help=_('IP addresses used by Nova metadata service')),
-    cfg.IntOpt('nova_metadata_port',
-               default=8775,
-               min=1, max=65535,
-               help=_("TCP Port used by Nova metadata server")),
+    cfg.PortOpt('nova_metadata_port',
+                default=8775,
+                help=_("TCP Port used by Nova metadata server")),
     cfg.StrOpt('metadata_shared_secret',
                secret=True,
                help=_('Shared secret to sign metadata requests')),
+    cfg.BoolOpt('metadata_insecure',
+                default=True,
+                help=_('If True, the end to end connection for metadata '
+                       'service is not verified. If False, the default CA '
+                       'truststore is used for verification')),
+    cfg.StrOpt('metadata_nova_client_cert',
+               help=_('Client certificate for nova metadata api server')),
+    cfg.StrOpt('metadata_nova_client_priv_key',
+               help=_('Private key of client certificate')),
     cfg.BoolOpt('spoofguard_enabled',
                 default=True,
                 help=_("If True then plugin will use NSXV spoofguard "
@@ -322,22 +377,29 @@ nsxv_opts = [
     cfg.IntOpt('dhcp_lease_time',
                default=86400,
                help=_('DHCP default lease time.')),
-    cfg.StrOpt('locking_coordinator_url',
-               help=_('A URL to a locking mechanism coordinator')),
     cfg.BoolOpt('metadata_initializer',
                 default=True,
                 help=_("If True, the server instance will attempt to "
                        "initialize the metadata infrastructure")),
+    cfg.ListOpt('metadata_service_allowed_ports',
+                help=_('List of tcp ports, to be allowed access to the '
+                       'metadata proxy, in addition to the default '
+                       '80,443,8775 tcp ports')),
     cfg.BoolOpt('edge_ha',
                 default=False,
                 help=_("Enable HA for NSX Edges")),
     cfg.StrOpt('exclusive_router_appliance_size',
                default="compact",
+               choices=routersize.VALID_EDGE_SIZES,
                help=_("Edge appliance size to be used for creating exclusive "
-                      "router. Valid values: ['compact', 'large', 'xlarge', "
-                      "'quadlarge']. This edge_appliance_size will be picked "
-                      "up if --router-size parameter is not specified while "
-                      "doing neutron router-create")),
+                      "router. This edge_appliance_size will be picked up if "
+                      "--router-size parameter is not specified while doing "
+                      "neutron router-create")),
+    cfg.ListOpt('nameservers',
+                default=[],
+                help=_('List of nameservers to configure for the DHCP binding '
+                       'entries. These will be used if there are no '
+                       'nameservers defined on the subnet.')),
 ]
 
 # Register the configuration options
@@ -350,14 +412,6 @@ cfg.CONF.register_opts(base_opts, group="NSX")
 cfg.CONF.register_opts(sync_opts, group="NSX_SYNC")
 
 
-def validate_config_options():
-    if cfg.CONF.NSX.replication_mode not in (ReplicationModes.SERVICE,
-                                             ReplicationModes.SOURCE):
-        error = (_("Invalid replication_mode: %s") %
-                 cfg.CONF.NSX.replication_mode)
-        raise nsx_exc.NsxPluginException(err_msg=error)
-
-
 def validate_nsxv_config_options():
     if (cfg.CONF.nsxv.manager_uri is None or
         cfg.CONF.nsxv.user is None or
