@@ -39,6 +39,16 @@ ROUTER_TYPE_TIER1 = "TIER1"
 
 ROUTER_TYPES = [ROUTER_TYPE_TIER0, ROUTER_TYPE_TIER1]
 
+LROUTERPORT_UPLINK = "LogicalRouterUplinkPort"
+LROUTERPORT_DOWNLINK = "LogicalRouterDownLinkPort"
+LROUTERPORT_LINKONTIER0 = "LogicalRouterLinkPortOnTIER0"
+LROUTERPORT_LINKONTIER1 = "LogicalRouterLinkPortOnTIER1"
+
+LROUTER_TYPES = [LROUTERPORT_UPLINK,
+                 LROUTERPORT_DOWNLINK,
+                 LROUTERPORT_LINKONTIER0,
+                 LROUTERPORT_LINKONTIER1]
+
 # L2 agent vif type
 VIF_TYPE_DVS = 'dvs'
 
