@@ -15,21 +15,21 @@
 
 import random
 
+from neutron_lib import constants
+from neutron_lib import exceptions
 from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_service import loopingcall
 from oslo_utils import timeutils
 import six
 
-from neutron.common import constants
-from neutron.common import exceptions
 from neutron import context
 from neutron.db import external_net_db
 from neutron.db import l3_db
 from neutron.db import models_v2
 from neutron.extensions import l3
-from neutron.i18n import _LE, _LI, _LW
 
+from vmware_nsx._i18n import _, _LE, _LI, _LW
 from vmware_nsx.api_client import exception as api_exc
 from vmware_nsx.common import exceptions as nsx_exc
 from vmware_nsx.common import nsx_utils
@@ -60,7 +60,7 @@ class NsxCache(object):
     """
 
     def __init__(self):
-        # Maps a uuid to the dict containing it
+        # Maps an uuid to the dict containing it
         self._uuid_dict_mappings = {}
         # Dicts for NSX cached resources
         self._lswitches = {}
@@ -110,7 +110,7 @@ class NsxCache(object):
                 resources[item_id]['hit'] = True
                 resources[item_id]['changed'] = True
                 resources[item_id]['data'] = item
-                # add a uuid to dict mapping for easy retrieval
+                # add an uuid to dict mapping for easy retrieval
                 # with __getitem__
                 self._uuid_dict_mappings[item_id] = resources
                 LOG.debug("Added item %s to NSX object cache", item_id)
@@ -406,8 +406,8 @@ class NsxSynchronizer():
                 neutron_router_mappings[neutron_router_id] = (
                     self._nsx_cache[lr_uuid])
             else:
-                LOG.warn(_LW("Unable to find Neutron router id for "
-                             "NSX logical router: %s"), lr_uuid)
+                LOG.warning(_LW("Unable to find Neutron router id for "
+                                "NSX logical router: %s"), lr_uuid)
         # Fetch neutron routers from database
         filters = ({} if scan_missing else
                    {'id': neutron_router_mappings.keys()})
@@ -541,16 +541,16 @@ class NsxSynchronizer():
             # be emitted.
             num_requests = page_size / (MAX_PAGE_SIZE + 1) + 1
             if num_requests > 1:
-                LOG.warn(_LW("Requested page size is %(cur_chunk_size)d. "
-                             "It might be necessary to do %(num_requests)d "
-                             "round-trips to NSX for fetching data. Please "
-                             "tune sync parameters to ensure chunk size "
-                             "is less than %(max_page_size)d"),
-                         {'cur_chunk_size': page_size,
-                          'num_requests': num_requests,
-                          'max_page_size': MAX_PAGE_SIZE})
+                LOG.warning(_LW("Requested page size is %(cur_chunk_size)d. "
+                                "It might be necessary to do %(num_requests)d "
+                                "round-trips to NSX for fetching data. Please "
+                                "tune sync parameters to ensure chunk size "
+                                "is less than %(max_page_size)d"),
+                            {'cur_chunk_size': page_size,
+                             'num_requests': num_requests,
+                             'max_page_size': MAX_PAGE_SIZE})
             # Only the first request might return the total size,
-            # subsequent requests will definetely not
+            # subsequent requests will definitely not
             results, cursor, total_size = nsxlib.get_single_query_page(
                 uri, self._cluster, cursor,
                 min(page_size, MAX_PAGE_SIZE))
