@@ -15,20 +15,25 @@
 #    under the License.
 
 import neutron.db.api as db
+from neutron.plugins.common import constants as neutron_const
+
+import decorator
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from oslo_utils import excutils
 import six
+from sqlalchemy import func
 from sqlalchemy.orm import exc
 from sqlalchemy.sql import expression as expr
 
-from neutron.i18n import _, _LE
-
+from vmware_nsx._i18n import _, _LE, _LW
 from vmware_nsx.common import exceptions as nsx_exc
 from vmware_nsx.common import nsxv_constants
 from vmware_nsx.db import nsxv_models
+from vmware_nsx.extensions import dns_search_domain as ext_dns_search_domain
 from vmware_nsx.plugins.nsx_v.vshield.common import constants
 
+NsxvEdgeDhcpStaticBinding = nsxv_models.NsxvEdgeDhcpStaticBinding
 LOG = logging.getLogger(__name__)
 
 
@@ -61,21 +66,50 @@ def add_nsxv_router_binding(session, router_id, vse_id, lswitch_id, status,
     return binding
 
 
+@decorator.decorator
+def warn_on_binding_status_error(f, *args, **kwargs):
+    result = f(*args, **kwargs)
+    if result is None:
+        return
+    # we support functions that return a single entry or a list
+    if type(result) == list:
+        bindings = result
+    else:
+        bindings = [result]
+
+    for binding in bindings:
+        if binding and binding['status'] == neutron_const.ERROR:
+            LOG.warning(_LW("Found NSXV router binding entry with status "
+                            "%(status)s: router %(router)s, "
+                            "edge %(edge)s, lswitch %(lswitch)s, "
+                            "status description: %(desc)s "),
+                        {'status': binding['status'],
+                         'router': binding['router_id'],
+                         'edge': binding['edge_id'],
+                         'lswitch': binding['lswitch_id'],
+                         'desc': binding['status_description']})
+    return result
+
+
+@warn_on_binding_status_error
 def get_nsxv_router_binding(session, router_id):
     return session.query(nsxv_models.NsxvRouterBinding).filter_by(
         router_id=router_id).first()
 
 
+@warn_on_binding_status_error
 def get_nsxv_router_binding_by_edge(session, edge_id):
     return session.query(nsxv_models.NsxvRouterBinding).filter_by(
         edge_id=edge_id).first()
 
 
+@warn_on_binding_status_error
 def get_nsxv_router_bindings_by_edge(session, edge_id):
     return session.query(nsxv_models.NsxvRouterBinding).filter_by(
         edge_id=edge_id).all()
 
 
+@warn_on_binding_status_error
 def get_nsxv_router_bindings(session, filters=None,
                              like_filters=None):
     session = db.get_session()
@@ -161,7 +195,7 @@ def clean_edge_vnic_binding(session, edge_id):
 
 
 def allocate_edge_vnic(session, edge_id, network_id):
-    """Allocate an avaliable edge vnic to network."""
+    """Allocate an available edge vnic to network."""
 
     with session.begin(subtransactions=True):
         bindings = (session.query(nsxv_models.NsxvEdgeVnicBinding).
@@ -179,7 +213,7 @@ def allocate_edge_vnic(session, edge_id, network_id):
 
 
 def allocate_edge_vnic_with_tunnel_index(session, edge_id, network_id):
-    """Allocate an avaliable edge vnic with tunnel index to network."""
+    """Allocate an available edge vnic with tunnel index to network."""
 
     # TODO(berlin): temporary solution to let metadata and dhcp use
     # different vnics
@@ -267,12 +301,25 @@ def get_edge_dhcp_static_binding(session, edge_id, mac_address):
         edge_id=edge_id, mac_address=mac_address).first()
 
 
+def get_dhcp_static_bindings_by_edge(session, edge_id):
+    return session.query(nsxv_models.NsxvEdgeDhcpStaticBinding).filter_by(
+        edge_id=edge_id).all()
+
+
 def delete_edge_dhcp_static_binding(session, edge_id, mac_address):
     with session.begin(subtransactions=True):
         session.query(nsxv_models.NsxvEdgeDhcpStaticBinding).filter_by(
             edge_id=edge_id, mac_address=mac_address).delete()
 
 
+def get_nsxv_dhcp_bindings_count_per_edge(session):
+    return (
+        session.query(
+            NsxvEdgeDhcpStaticBinding.edge_id,
+            func.count(NsxvEdgeDhcpStaticBinding.mac_address)).group_by(
+            NsxvEdgeDhcpStaticBinding.edge_id).all())
+
+
 def clean_edge_dhcp_static_bindings_by_edge(session, edge_id):
     with session.begin(subtransactions=True):
         session.query(nsxv_models.NsxvEdgeDhcpStaticBinding).filter_by(
@@ -507,6 +554,15 @@ def get_spoofguard_policy_id(session, network_id):
                   network_id)
 
 
+def get_nsxv_spoofguard_policy_network_mappings(session, filters=None,
+                                                like_filters=None):
+    session = db.get_session()
+    query = session.query(nsxv_models.NsxvSpoofGuardPolicyNetworkMapping)
+    return _apply_filters_to_query(
+               query, nsxv_models.NsxvSpoofGuardPolicyNetworkMapping,
+               filters, like_filters).all()
+
+
 def add_vdr_dhcp_binding(session, vdr_router_id, dhcp_edge_id):
     with session.begin(subtransactions=True):
         binding = nsxv_models.NsxvVdrDhcpBinding(vdr_router_id=vdr_router_id,
@@ -536,3 +592,185 @@ def get_vdr_dhcp_binding_by_vdr(session, vdr_router_id):
 def delete_vdr_dhcp_binding(session, vdr_router_id):
     return (session.query(nsxv_models.NsxvVdrDhcpBinding).
             filter_by(vdr_router_id=vdr_router_id).delete())
+
+
+def add_nsxv_lbaas_loadbalancer_binding(
+        session, loadbalancer_id, edge_id, edge_fw_rule_id, vip_address):
+    with session.begin(subtransactions=True):
+        binding = nsxv_models.NsxvLbaasLoadbalancerBinding(
+            loadbalancer_id=loadbalancer_id,
+            edge_id=edge_id,
+            edge_fw_rule_id=edge_fw_rule_id,
+            vip_address=vip_address)
+        session.add(binding)
+    return binding
+
+
+def get_nsxv_lbaas_loadbalancer_binding(session, loadbalancer_id):
+    try:
+        return session.query(
+            nsxv_models.NsxvLbaasLoadbalancerBinding).filter_by(
+            loadbalancer_id=loadbalancer_id).one()
+    except exc.NoResultFound:
+        return
+
+
+def get_nsxv_lbaas_loadbalancer_binding_by_edge(session, edge_id):
+    return session.query(
+        nsxv_models.NsxvLbaasLoadbalancerBinding).filter_by(
+        edge_id=edge_id).all()
+
+
+def del_nsxv_lbaas_loadbalancer_binding(session, loadbalancer_id):
+    return (session.query(nsxv_models.NsxvLbaasLoadbalancerBinding).
+            filter_by(loadbalancer_id=loadbalancer_id).delete())
+
+
+def add_nsxv_lbaas_listener_binding(session, loadbalancer_id, listener_id,
+                                    app_profile_id, vse_id):
+    with session.begin(subtransactions=True):
+        binding = nsxv_models.NsxvLbaasListenerBinding(
+            loadbalancer_id=loadbalancer_id,
+            listener_id=listener_id,
+            app_profile_id=app_profile_id,
+            vse_id=vse_id)
+        session.add(binding)
+    return binding
+
+
+def get_nsxv_lbaas_listener_binding(session, loadbalancer_id, listener_id):
+    try:
+        return session.query(
+            nsxv_models.NsxvLbaasListenerBinding).filter_by(
+            loadbalancer_id=loadbalancer_id, listener_id=listener_id).one()
+    except exc.NoResultFound:
+        return
+
+
+def del_nsxv_lbaas_listener_binding(session, loadbalancer_id, listener_id):
+    return (session.query(nsxv_models.NsxvLbaasListenerBinding).
+            filter_by(loadbalancer_id=loadbalancer_id,
+                      listener_id=listener_id).delete())
+
+
+def add_nsxv_lbaas_pool_binding(session, loadbalancer_id, listener_id,
+                                pool_id, edge_pool_id):
+    with session.begin(subtransactions=True):
+        binding = nsxv_models.NsxvLbaasPoolBinding(
+            loadbalancer_id=loadbalancer_id,
+            listener_id=listener_id,
+            pool_id=pool_id,
+            edge_pool_id=edge_pool_id)
+        session.add(binding)
+    return binding
+
+
+def get_nsxv_lbaas_pool_binding(session, loadbalancer_id, listener_id,
+                                pool_id):
+    try:
+        return session.query(
+            nsxv_models.NsxvLbaasPoolBinding).filter_by(
+            loadbalancer_id=loadbalancer_id,
+            listener_id=listener_id,
+            pool_id=pool_id).one()
+    except exc.NoResultFound:
+        return
+
+
+def del_nsxv_lbaas_pool_binding(session, loadbalancer_id, listener_id,
+                                pool_id):
+    return (session.query(nsxv_models.NsxvLbaasPoolBinding).
+            filter_by(loadbalancer_id=loadbalancer_id,
+                      listener_id=listener_id,
+                      pool_id=pool_id).delete())
+
+
+def add_nsxv_lbaas_monitor_binding(session, loadbalancer_id, listener_id,
+                                   pool_id, hm_id, edge_id, edge_mon_id):
+    with session.begin(subtransactions=True):
+        binding = nsxv_models.NsxvLbaasMonitorBinding(
+            loadbalancer_id=loadbalancer_id,
+            listener_id=listener_id,
+            pool_id=pool_id,
+            hm_id=hm_id,
+            edge_id=edge_id,
+            edge_mon_id=edge_mon_id)
+        session.add(binding)
+    return binding
+
+
+def get_nsxv_lbaas_monitor_binding(session, loadbalancer_id, listener_id,
+                                   pool_id, hm_id, edge_id):
+    try:
+        return session.query(
+            nsxv_models.NsxvLbaasMonitorBinding).filter_by(
+            loadbalancer_id=loadbalancer_id,
+            listener_id=listener_id,
+            pool_id=pool_id,
+            hm_id=hm_id,
+            edge_id=edge_id).one()
+    except exc.NoResultFound:
+        return
+
+
+def del_nsxv_lbaas_monitor_binding(session, loadbalancer_id, listener_id,
+                                   pool_id, hm_id, edge_id):
+    return (session.query(nsxv_models.NsxvLbaasMonitorBinding).
+            filter_by(loadbalancer_id=loadbalancer_id,
+                      listener_id=listener_id,
+                      pool_id=pool_id,
+                      hm_id=hm_id,
+                      edge_id=edge_id).delete())
+
+
+def add_nsxv_lbaas_certificate_binding(session, cert_id, edge_id,
+                                       edge_cert_id):
+    with session.begin(subtransactions=True):
+        binding = nsxv_models.NsxvLbaasCertificateBinding(
+            cert_id=cert_id,
+            edge_id=edge_id,
+            edge_cert_id=edge_cert_id)
+        session.add(binding)
+    return binding
+
+
+def get_nsxv_lbaas_certificate_binding(session, cert_id, edge_id):
+    try:
+        return session.query(
+            nsxv_models.NsxvLbaasCertificateBinding).filter_by(
+            cert_id=cert_id,
+            edge_id=edge_id).one()
+    except exc.NoResultFound:
+        return
+
+
+def del_nsxv_lbaas_certificate_binding(session, cert_id, edge_id):
+    return (session.query(nsxv_models.NsxvLbaasCertificateBinding).
+            filter_by(cert_id=cert_id,
+                      edge_id=edge_id).delete())
+
+
+def add_nsxv_subnet_ext_attributes(session, subnet_id, dns_search_domain):
+    with session.begin(subtransactions=True):
+        binding = nsxv_models.NsxvSubnetExtAttributes(
+            subnet_id=subnet_id,
+            dns_search_domain=dns_search_domain)
+        session.add(binding)
+    return binding
+
+
+def get_nsxv_subnet_ext_attributes(session, subnet_id):
+    try:
+        return session.query(
+            nsxv_models.NsxvSubnetExtAttributes).filter_by(
+            subnet_id=subnet_id).one()
+    except exc.NoResultFound:
+        return
+
+
+def update_nsxv_subnet_ext_attributes(session, subnet_id, dns_search_domain):
+    with session.begin(subtransactions=True):
+        binding = (session.query(nsxv_models.NsxvSubnetExtAttributes).
+                   filter_by(subnet_id=subnet_id).one())
+        binding[ext_dns_search_domain.DNS_SEARCH_DOMAIN] = dns_search_domain
+    return binding
