@@ -20,12 +20,12 @@ from oslo_utils import excutils
 
 from neutron.api import extensions as neutron_extensions
 from neutron.api.v2 import attributes as attr
-from neutron.common import exceptions as n_exc
 from neutron.db import agentschedulers_db
 from neutron.db import allowedaddresspairs_db as addr_pair_db
 from neutron.db import db_base_plugin_v2
 from neutron.db import external_net_db
 from neutron.db import l3_db
+from neutron.db import models_v2
 from neutron.db import portbindings_db
 from neutron.db import portsecurity_db
 from neutron.db import securitygroups_db
@@ -35,17 +35,19 @@ from neutron.extensions import portbindings as pbin
 from neutron.extensions import portsecurity as psec
 from neutron.extensions import providernet as pnet
 from neutron.extensions import securitygroup as ext_sg
-from neutron.i18n import _LE, _LW
 from neutron.plugins.common import constants
 from neutron.plugins.common import utils
+from neutron.quota import resource_registry
+from neutron_lib import exceptions as n_exc
 
 import vmware_nsx
+from vmware_nsx._i18n import _, _LE, _LW
 from vmware_nsx.common import config  # noqa
 from vmware_nsx.common import exceptions as nsx_exc
 from vmware_nsx.common import nsx_constants
 from vmware_nsx.common import utils as c_utils
 from vmware_nsx.db import db as nsx_db
-from vmware_nsx import dhcpmeta_modes
+from vmware_nsx.dhcp_meta import modes as dhcpmeta_modes
 from vmware_nsx.dvs import dvs
 from vmware_nsx.dvs import dvs_utils
 
@@ -76,12 +78,19 @@ class NsxDvsV2(addr_pair_db.AllowedAddressPairsMixin,
     __native_pagination_support = True
     __native_sorting_support = True
 
+    @resource_registry.tracked_resources(
+        network=models_v2.Network,
+        port=models_v2.Port,
+        subnet=models_v2.Subnet,
+        subnetpool=models_v2.SubnetPool,
+        security_group=securitygroups_db.SecurityGroup,
+        security_group_rule=securitygroups_db.SecurityGroupRule)
     def __init__(self):
         super(NsxDvsV2, self).__init__()
-        config.validate_config_options()
         LOG.debug('Driver support: DVS: %s' % dvs_utils.dvs_is_enabled())
         neutron_extensions.append_api_extensions_path(
             [vmware_nsx.NSX_EXT_PATH])
+        self.cfg_group = 'dvs'  # group name for dvs section in nsx.ini
         self._dvs = dvs.DvsManager()
 
         # Common driver code
@@ -161,6 +170,12 @@ class NsxDvsV2(addr_pair_db.AllowedAddressPairsMixin,
         new_net[pnet.NETWORK_TYPE] = net_data.get(pnet.NETWORK_TYPE)
         new_net[pnet.PHYSICAL_NETWORK] = 'dvs'
         new_net[pnet.SEGMENTATION_ID] = vlan_tag
+
+        # this extra lookup is necessary to get the
+        # latest db model for the extension functions
+        net_model = self._get_network(context, net_data['id'])
+        self._apply_dict_extend_functions('networks', new_net, net_model)
+
         self.handle_network_dhcp_access(context, new_net,
                                         action='create_network')
         return new_net
@@ -241,7 +256,8 @@ class NsxDvsV2(addr_pair_db.AllowedAddressPairsMixin,
                     limit, marker, page_reverse))
             for net in networks:
                 self._extend_network_dict_provider(context, net)
-        return [self._fields(network, fields) for network in networks]
+        return (networks if not fields else
+                [self._fields(network, fields) for network in networks])
 
     def update_network(self, context, id, network):
         raise nsx_exc.NsxPluginException(
@@ -301,6 +317,11 @@ class NsxDvsV2(addr_pair_db.AllowedAddressPairsMixin,
         # DB Operation is complete, perform DVS operation
         port_data = port['port']
 
+        # this extra lookup is necessary to get the
+        # latest db model for the extension functions
+        port_model = self._get_port(context, port_data['id'])
+        self._apply_dict_extend_functions('ports', port_data, port_model)
+
         self.handle_port_dhcp_access(context, port_data, action='create_port')
         return port_data
 
