@@ -16,10 +16,12 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 
 from neutron.api.v2 import attributes as attr
-from neutron.common import exceptions as n_exc
 from neutron.db import l3_db
-from neutron.i18n import _LE
 
+from neutron_lib import exceptions as n_exc
+
+from vmware_nsx._i18n import _LE
+from vmware_nsx.common import locking
 from vmware_nsx.db import nsxv_db
 from vmware_nsx.plugins.nsx_v.drivers import (
     abstract_router_driver as router_driver)
@@ -37,6 +39,10 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
     def get_type(self):
         return "distributed"
 
+    def _get_edge_id(self, context, router_id):
+        binding = nsxv_db.get_nsxv_router_binding(context.session, router_id)
+        return binding.get('edge_id')
+
     def _update_routes_on_plr(self, context, router_id, plr_id, newnexthop):
         lswitch_id = edge_utils.get_internal_lswitch_id_of_plr_tlr(
             context, router_id)
@@ -98,8 +104,8 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
         super(nsx_v.NsxVPluginV2, self.plugin).update_router(
             context, router_id, router)
         if gw_info != attr.ATTR_NOT_SPECIFIED:
-            self._update_router_gw_info(context, router_id, gw_info,
-                                        is_routes_update)
+            self.plugin._update_router_gw_info(context, router_id, gw_info,
+                                               is_routes_update)
         elif is_routes_update:
             # here is used to handle routes which tenant updates.
             router_db = self.plugin._get_router(context, router_id)
@@ -107,7 +113,7 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
                 context, router_db)[2]
             self.plugin._update_subnets_and_dnat_firewall(context, router_db)
             md_gw_data = self._get_metadata_gw_data(context, router_id)
-            self.update_routes(context, router_id, nexthop, md_gw_data)
+            self._update_routes(context, router_id, nexthop, md_gw_data)
         if 'admin_state_up' in r:
             self.plugin._update_router_admin_state(
                 context, router_id, self.get_type(), r['admin_state_up'])
@@ -118,9 +124,17 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
 
     def update_routes(self, context, router_id, newnexthop,
                       metadata_gateway=None):
+        with locking.LockManager.get_lock(self._get_edge_id(context,
+                                                            router_id)):
+            self._update_routes(context, router_id, newnexthop,
+                                metadata_gateway)
+
+    def _update_routes(self, context, router_id, newnexthop,
+                      metadata_gateway=None):
         plr_id = self.edge_manager.get_plr_by_tlr_id(context, router_id)
         if plr_id:
-            self._update_routes_on_plr(context, router_id, plr_id, newnexthop)
+            self._update_routes_on_plr(context, router_id, plr_id,
+                                       newnexthop)
             self._update_routes_on_tlr(context, router_id,
                                        metadata_gateway=metadata_gateway)
         else:
@@ -128,7 +142,8 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
                                        metadata_gateway=metadata_gateway)
 
     def _update_router_gw_info(self, context, router_id, info,
-                               is_routes_update=False):
+                               is_routes_update=False,
+                               force_update=False):
         router = self.plugin._get_router(context, router_id)
         org_ext_net_id = router.gw_port_id and router.gw_port.network_id
         org_enable_snat = router.enable_snat
@@ -183,9 +198,31 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
 
         # update static routes in all
         md_gw_data = self._get_metadata_gw_data(context, router_id)
-        self.update_routes(context, router_id, newnexthop, md_gw_data)
+        self._update_routes(context, router_id, newnexthop, md_gw_data)
+
+    def _validate_multiple_subnets_routers(self, context, router_id,
+                                           interface_info):
+        _nsxv_plugin = self.plugin
+        net_id = _nsxv_plugin._get_interface_info_net_id(context,
+                                                         interface_info)
+
+        port_filters = {'device_owner': [l3_db.DEVICE_OWNER_ROUTER_INTF],
+                        'network_id': [net_id]}
+        intf_ports = _nsxv_plugin.get_ports(context.elevated(),
+                                            filters=port_filters)
+        router_ids = [port['device_id'] for port in intf_ports]
+        dist_router_filters = {'id': router_ids,
+                               'distributed': [True]}
+        dist_routers = _nsxv_plugin.get_routers(context, dist_router_filters)
+        if len(dist_routers) > 0:
+            err_msg = _("network can only be attached to just one distributed "
+                        "router, the network is already attached to router "
+                        "%(router_id)s") % {'router_id': dist_routers[0]['id']}
+            raise n_exc.InvalidInput(error_message=err_msg)
 
     def add_router_interface(self, context, router_id, interface_info):
+        self._validate_multiple_subnets_routers(
+            context, router_id, interface_info)
         info = super(nsx_v.NsxVPluginV2, self.plugin).add_router_interface(
             context, router_id, interface_info)
 
@@ -194,47 +231,54 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
         network_id = subnet['network_id']
         address_groups = self.plugin._get_address_groups(
             context, router_id, network_id)
-        port = self.plugin.get_port(context, info['port_id'])
-        try:
-            edge_utils.add_vdr_internal_interface(self.nsx_v, context,
-                                                  router_id, network_id,
-                                                  address_groups,
-                                                  router_db.admin_state_up)
-        except n_exc.BadRequest:
-            with excutils.save_and_reraise_exception():
-                super(nsx_v.NsxVPluginV2, self.plugin).remove_router_interface(
-                    context, router_id, interface_info)
-        # Update edge's firewall rules to accept subnets flows.
-        self.plugin._update_subnets_and_dnat_firewall(context, router_db)
+        with locking.LockManager.get_lock(self._get_edge_id(context,
+                                                            router_id)):
+            port = self.plugin.get_port(context, info['port_id'])
+            try:
+                edge_utils.add_vdr_internal_interface(self.nsx_v, context,
+                                                      router_id, network_id,
+                                                      address_groups,
+                                                      router_db.admin_state_up)
+            except Exception:
+                with excutils.save_and_reraise_exception():
+                    super(nsx_v.NsxVPluginV2, self.plugin
+                          ).remove_router_interface(context,
+                                                    router_id,
+                                                    interface_info)
+            # Update edge's firewall rules to accept subnets flows.
+            self.plugin._update_subnets_and_dnat_firewall(context, router_db)
 
-        do_metadata = False
-        if self.plugin.metadata_proxy_handler:
-            for fixed_ip in port.get("fixed_ips", []):
-                if fixed_ip['ip_address'] == subnet['gateway_ip']:
-                    do_metadata = True
+            do_metadata = False
+            if self.plugin.metadata_proxy_handler:
+                for fixed_ip in port.get("fixed_ips", []):
+                    if fixed_ip['ip_address'] == subnet['gateway_ip']:
+                        do_metadata = True
 
-            if do_metadata:
-                self.edge_manager.configure_dhcp_for_vdr_network(
-                    context, network_id, router_id)
+                if do_metadata:
+                    self.edge_manager.configure_dhcp_for_vdr_network(
+                        context, network_id, router_id)
 
-        if router_db.gw_port and router_db.enable_snat:
-            plr_id = self.edge_manager.get_plr_by_tlr_id(context, router_id)
-            self.plugin._update_nat_rules(context, router_db, plr_id)
-            # Open firewall flows on plr
-            self.plugin._update_subnets_and_dnat_firewall(
-                context, router_db, router_id=plr_id)
-            # Update static routes of plr
-            nexthop = self.plugin._get_external_attachment_info(
-                context, router_db)[2]
-            if do_metadata:
-                md_gw_data = self._get_metadata_gw_data(context, router_id)
-            else:
-                md_gw_data = None
-            self.update_routes(context, router_id, nexthop, md_gw_data)
+            if router_db.gw_port:
+                plr_id = self.edge_manager.get_plr_by_tlr_id(context,
+                                                             router_id)
+                if router_db.enable_snat:
+                    self.plugin._update_nat_rules(context, router_db, plr_id)
 
-        elif do_metadata and self._metadata_cfg_required_after_port_add(
-                context, router_id, subnet):
-            self._metadata_route_update(context, router_id)
+                # Open firewall flows on plr
+                self.plugin._update_subnets_and_dnat_firewall(
+                    context, router_db, router_id=plr_id)
+                # Update static routes of plr
+                nexthop = self.plugin._get_external_attachment_info(
+                    context, router_db)[2]
+                if do_metadata:
+                    md_gw_data = self._get_metadata_gw_data(context, router_id)
+                else:
+                    md_gw_data = None
+                self._update_routes(context, router_id, nexthop, md_gw_data)
+
+            elif do_metadata and self._metadata_cfg_required_after_port_add(
+                    context, router_id, subnet):
+                self._metadata_route_update(context, router_id)
 
         return info
 
@@ -276,9 +320,9 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
                          'fixed_ips': {'subnet_id': [vdr_subnets[0]['id']]}},
                 fields=['fixed_ips'])
 
-            if(dhcp_ports
-               and dhcp_ports[0].get('fixed_ips')
-               and dhcp_ports[0]['fixed_ips'][0]):
+            if (dhcp_ports
+                and dhcp_ports[0].get('fixed_ips')
+                and dhcp_ports[0]['fixed_ips'][0]):
                 ip_subnet = dhcp_ports[0]['fixed_ips'][0]
                 ip_address = ip_subnet['ip_address']
                 network_id = self.plugin.get_subnet(
@@ -306,8 +350,8 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
                     subnet_id = port['fixed_ips'][0]['subnet_id']
                     port_subnet = self.plugin.get_subnet(
                         context, subnet_id)
-                    if(port_subnet['id'] != subnet['id']
-                       and port_subnet['enable_dhcp']):
+                    if (port_subnet['id'] != subnet['id']
+                        and port_subnet['enable_dhcp']):
                         # We already have a subnet which is connected to
                         # DHCP - hence no need to change the metadata route
                         return False
@@ -322,8 +366,8 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
         routes = edge_utils.get_routes(self.nsx_v, context, router_id)
 
         for route in routes:
-            if(route['destination'] == METADATA_CIDR
-               and subnet['network_id'] == route['network_id']):
+            if (route['destination'] == METADATA_CIDR
+                and subnet['network_id'] == route['network_id']):
 
                 # Metadata requests are transferred via this port
                 return True
@@ -336,67 +380,75 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
         subnet = self.plugin.get_subnet(context, info['subnet_id'])
         network_id = subnet['network_id']
 
-        if router_db.gw_port and router_db.enable_snat:
-            plr_id = self.edge_manager.get_plr_by_tlr_id(
-                context, router_id)
-            self.plugin._update_nat_rules(context, router_db, plr_id)
-            # Open firewall flows on plr
-            self.plugin._update_subnets_and_dnat_firewall(
-                context, router_db, router_id=plr_id)
-            # Update static routes of plr
-            nexthop = self.plugin._get_external_attachment_info(
-                context, router_db)[2]
-            md_gw_data = self._get_metadata_gw_data(context, router_id)
-            self.update_routes(context, router_id, nexthop, md_gw_data)
-            if (subnet['enable_dhcp'] and self.plugin.metadata_proxy_handler
-                and not md_gw_data):
-                # No more DHCP interfaces on VDR. Remove DHCP binding
-                nsxv_db.delete_vdr_dhcp_binding(context.session, router_id)
+        with locking.LockManager.get_lock(self._get_edge_id(context,
+                                                            router_id)):
+            if router_db.gw_port and router_db.enable_snat:
+                plr_id = self.edge_manager.get_plr_by_tlr_id(
+                    context, router_id)
+                self.plugin._update_nat_rules(context, router_db, plr_id)
+                # Open firewall flows on plr
+                self.plugin._update_subnets_and_dnat_firewall(
+                    context, router_db, router_id=plr_id)
+                # Update static routes of plr
+                nexthop = self.plugin._get_external_attachment_info(
+                    context, router_db)[2]
+                md_gw_data = self._get_metadata_gw_data(context, router_id)
+                self._update_routes(context, router_id, nexthop, md_gw_data)
+                if (subnet['enable_dhcp']
+                    and self.plugin.metadata_proxy_handler
+                    and not md_gw_data):
+                    # No more DHCP interfaces on VDR. Remove DHCP binding
+                    nsxv_db.delete_vdr_dhcp_binding(context.session, router_id)
 
-        # If DHCP is disabled, this remove cannot trigger metadata change
-        # as metadata is served via DHCP Edge
-        elif (subnet['enable_dhcp'] and self.plugin.metadata_proxy_handler and
-              self._metadata_cfg_required_after_port_remove(context, router_id,
-                                                            subnet)):
-            md_gw_data = self._metadata_route_update(context, router_id)
+            # If DHCP is disabled, this remove cannot trigger metadata change
+            # as metadata is served via DHCP Edge
+            elif (subnet['enable_dhcp']
+                  and self.plugin.metadata_proxy_handler):
+                md_gw_data = self._get_metadata_gw_data(context, router_id)
+                if self._metadata_cfg_required_after_port_remove(
+                    context, router_id, subnet):
+                    self._metadata_route_update(context, router_id)
 
-        self.plugin._update_subnets_and_dnat_firewall(context, router_db)
-        # Safly remove interface, VDR can have interface to only one subnet in
-        # a given network.
-        edge_utils.delete_interface(
-            self.nsx_v, context, router_id, network_id, dist=True)
+            self.plugin._update_subnets_and_dnat_firewall(context, router_db)
+            # Safly remove interface, VDR can have interface to only one subnet
+            # in a given network.
+            edge_utils.delete_interface(
+                self.nsx_v, context, router_id, network_id, dist=True)
 
-        # The network would be the last one attached to the VDR if md_gw_data
-        # is None. For such condition, we just keep network attached to the
-        # dhcp edge since the dhcp edge is a pure dhcp support edge now
-        if (self.plugin.metadata_proxy_handler and subnet['enable_dhcp'] and
-            md_gw_data):
-            # Detach network from VDR-dedicated DHCP Edge
-            vdr_dhcp_binding = nsxv_db.get_vdr_dhcp_binding_by_vdr(
-                context.session, router_id)
+            # The network would be the last one attached to the VDR if
+            # md_gw_data is None. For such condition, we just keep network
+            # attached to the dhcp edge since the dhcp edge is a pure dhcp
+            # support edge now
+            if (self.plugin.metadata_proxy_handler and subnet['enable_dhcp']
+                and md_gw_data):
+                # Detach network from VDR-dedicated DHCP Edge
+                vdr_dhcp_binding = nsxv_db.get_vdr_dhcp_binding_by_vdr(
+                    context.session, router_id)
 
-            # A case where we do not have a vdr_dhcp_binding indicates a DB
-            # inconsistency. We check for this anyway, in case that something
-            # is broken.
-            if vdr_dhcp_binding:
-                self.edge_manager.reset_sysctl_rp_filter_for_vdr_dhcp(
-                    context, vdr_dhcp_binding['dhcp_edge_id'], network_id)
+                # A case where we do not have a vdr_dhcp_binding indicates a DB
+                # inconsistency. We check for this anyway, in case that
+                # something is broken.
+                if vdr_dhcp_binding:
+                    self.edge_manager.reset_sysctl_rp_filter_for_vdr_dhcp(
+                        context, vdr_dhcp_binding['dhcp_edge_id'], network_id)
 
-                self.edge_manager.remove_network_from_dhcp_edge(
-                    context, network_id, vdr_dhcp_binding['dhcp_edge_id'])
-            else:
-                LOG.error(_LE('VDR DHCP binding is missing for %s'), router_id)
+                    self.edge_manager.remove_network_from_dhcp_edge(
+                        context, network_id, vdr_dhcp_binding['dhcp_edge_id'])
+                else:
+                    LOG.error(_LE('VDR DHCP binding is missing for %s'),
+                              router_id)
 
-            # Reattach to regular DHCP Edge
-            self.edge_manager.create_dhcp_edge_service(
-                context, network_id, subnet)
+                # Reattach to regular DHCP Edge
+                self.edge_manager.create_dhcp_edge_service(
+                    context, network_id, subnet)
 
-            address_groups = self.plugin._create_network_dhcp_address_group(
-                context, network_id)
-            self.edge_manager.update_dhcp_edge_service(
-                context, network_id, address_groups=address_groups)
+                address_groups = (
+                    self.plugin._create_network_dhcp_address_group(context,
+                                                                   network_id))
+                self.edge_manager.update_dhcp_edge_service(
+                    context, network_id, address_groups=address_groups)
 
-        return info
+            return info
 
     def _update_edge_router(self, context, router_id):
         router = self.plugin._get_router(context.elevated(), router_id)
@@ -406,3 +458,28 @@ class RouterDistributedDriver(router_driver.RouterBaseDriver):
         self.plugin._update_nat_rules(context, router, router_id=plr_id)
         self.plugin._update_subnets_and_dnat_firewall(context, router,
                                                       router_id=plr_id)
+
+    def update_router_interface_ip(self, context, router_id,
+                                   port_id, int_net_id,
+                                   old_ip, new_ip, subnet_mask):
+        """Update the fixed ip of a distributed router interface. """
+        router = self.plugin._get_router(context, router_id)
+        if port_id == router.gw_port_id:
+            # external port / Uplink
+            plr_id = self.edge_manager.get_plr_by_tlr_id(context, router_id)
+            edge_id = self._get_edge_id_or_raise(context, plr_id)
+            self.edge_manager.update_interface_addr(
+                context, edge_id, old_ip, new_ip, subnet_mask, is_uplink=True)
+            # Also update the nat rules
+            self.plugin._update_nat_rules(context, router, plr_id)
+        else:
+            # Internal port:
+            # get the edge-id of this router
+            edge_id = self._get_edge_id_or_raise(context, router_id)
+            # Get the vnic index
+            edge_vnic_binding = nsxv_db.get_edge_vnic_binding(
+                context.session, edge_id, int_net_id)
+            vnic_index = edge_vnic_binding.vnic_index
+            self.edge_manager.update_vdr_interface_addr(
+                context, edge_id, vnic_index, old_ip, new_ip,
+                subnet_mask)
