@@ -18,7 +18,10 @@ from oslo_config import cfg
 from neutron.api.v2 import attributes as attr
 from neutron.db import l3_db
 from neutron.db import models_v2
+from neutron_lib import exceptions as n_exc
 from oslo_log import log as logging
+
+from vmware_nsx._i18n import _
 from vmware_nsx.common import exceptions as nsx_exc
 from vmware_nsx.common import locking
 from vmware_nsx.db import nsxv_db
@@ -33,8 +36,6 @@ from vmware_nsx.plugins.nsx_v.vshield import edge_utils
 
 LOG = logging.getLogger(__name__)
 
-NSXV_ROUTER_RECONFIG = "nsxv_router_reconfig"
-
 
 class RouterSharedDriver(router_driver.RouterBaseDriver):
 
@@ -45,25 +46,28 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                       appliance_size=None, allow_metadata=True):
         pass
 
+    def _validate_no_routes(self, router):
+        if (attr.is_attr_set(router.get('routes')) and
+            len(router['routes']) > 0):
+            msg = _("Cannot configure static routes on a shared router")
+            raise n_exc.InvalidInput(error_message=msg)
+
     def update_router(self, context, router_id, router):
         r = router['router']
-        is_routes_update = True if 'routes' in r else False
+        self._validate_no_routes(r)
         edge_id = edge_utils.get_router_edge_id(context, router_id)
         if not edge_id:
             return super(nsx_v.NsxVPluginV2, self.plugin).update_router(
                 context, router_id, router)
         else:
-            with locking.LockManager.get_lock(
-                    str(edge_id), lock_file_prefix=NSXV_ROUTER_RECONFIG):
+            with locking.LockManager.get_lock(str(edge_id)):
                 gw_info = self.plugin._extract_external_gw(
                     context, router, is_extract=True)
                 super(nsx_v.NsxVPluginV2, self.plugin).update_router(
                     context, router_id, router)
-                self.update_routes(context, router_id, None)
-            # here is used to handle routes which tenant updates.
+
             if gw_info != attr.ATTR_NOT_SPECIFIED:
-                self._update_router_gw_info(context, router_id, gw_info,
-                                            is_routes_update)
+                self.plugin._update_router_gw_info(context, router_id, gw_info)
             if 'admin_state_up' in r:
                 # If router was deployed on a different edge then
                 # admin-state-up is already updated on the new edge.
@@ -75,34 +79,120 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                                                            r['admin_state_up'])
             return self.plugin.get_router(context, router_id)
 
+    def detach_router(self, context, router_id, router):
+        LOG.debug("Detach shared router id %s", router_id)
+        # if it is the last shared router on this adge - add it to the pool
+        edge_id = edge_utils.get_router_edge_id(context, router_id)
+        if not edge_id:
+            return
+
+        self._remove_router_services_on_edge(context, router_id)
+        self._unbind_router_on_edge(context, router_id)
+
+    def attach_router(self, context, router_id, router, appliance_size=None):
+        # find the right place to add, and create a new one if necessary
+        router_db = self.plugin._get_router(context, router_id)
+        self._bind_router_on_available_edge(
+            context, router_id, router_db.admin_state_up)
+        edge_id = edge_utils.get_router_edge_id(context, router_id)
+        LOG.debug("Shared router %s attached to edge %s", router_id, edge_id)
+        with locking.LockManager.get_lock(str(edge_id)):
+            self._add_router_services_on_available_edge(context, router_id)
+
     def delete_router(self, context, router_id):
         pass
 
-    def _update_routes_on_routers(self, context, target_router_id, router_ids):
+    def _get_router_routes(self, context, router_id):
+        return self.plugin._get_extra_routes_by_router_id(
+            context, router_id)
+
+    def _get_router_next_hop(self, context, router_id):
+        router_qry = context.session.query(l3_db.Router)
+        router_db = router_qry.filter_by(id=router_id).one()
+        return self.plugin._get_external_attachment_info(
+            context, router_db)[2]
+
+    def _update_routes_on_routers(self, context, target_router_id, router_ids,
+                                  only_if_target_routes=False):
+        if only_if_target_routes:
+            # First check if the target router has any routes or next hop
+            # If not - it means that nothing changes so we can skip this
+            # backend call
+            target_routes = self._get_router_routes(context, target_router_id)
+            target_next_hop = self._get_router_next_hop(
+                context, target_router_id)
+            if not target_routes and not target_next_hop:
+                LOG.debug("_update_routes_on_routers skipped since router %s "
+                          "has no routes", target_router_id)
+                return
+
         nexthop = None
         all_routes = []
         for router_id in router_ids:
-            routes = self.plugin._get_extra_routes_by_router_id(
-                context, router_id)
+            routes = self._get_router_routes(context, router_id)
             filters = {'device_id': [router_id]}
             ports = self.plugin.get_ports(context.elevated(), filters)
             self.plugin._add_network_info_for_routes(context, routes, ports)
             all_routes.extend(routes)
             if not nexthop:
-                router_qry = context.session.query(l3_db.Router)
-                router_db = router_qry.filter_by(id=router_id).one()
-                router_nexthop = self.plugin._get_external_attachment_info(
-                    context, router_db)[2]
+                router_nexthop = self._get_router_next_hop(context,
+                                                           router_id)
                 if router_nexthop:
                     nexthop = router_nexthop
         # TODO(berlin) do rollback op.
         edge_utils.update_routes(self.nsx_v, context, target_router_id,
                                  all_routes, nexthop)
 
+    # return a dic of each router -> list of vnics from the other routers
+    def _get_all_routers_vnic_indices(self, context, router_ids):
+
+        all_vnic_indices = {}
+        if len(router_ids) < 1:
+            # there are no routers
+            return all_vnic_indices
+
+        intf_ports = self.plugin.get_ports(
+            context.elevated(),
+            filters={'device_owner': [l3_db.DEVICE_OWNER_ROUTER_INTF]})
+
+        edge_id = edge_utils.get_router_edge_id(context, router_ids[0])
+        edge_vnic_bindings = nsxv_db.get_edge_vnic_bindings_by_edge(
+            context.session, edge_id)
+
+        for this_router_id in router_ids:
+            # get networks IDs for this router
+            router_net_ids = list(
+                set([port['network_id']
+                     for port in intf_ports
+                     if port['device_id'] == this_router_id]))
+
+            # get vnic index for each network
+            vnic_indices = []
+            for net_id in router_net_ids:
+                vnic_indices.extend([edge_vnic_binding.vnic_index
+                                     for edge_vnic_binding
+                                     in edge_vnic_bindings
+                                     if edge_vnic_binding.network_id == net_id
+                                     ])
+
+            # make sure the list is unique:
+            vnic_indices = list(set(vnic_indices))
+            # add to the result dict
+            all_vnic_indices[this_router_id] = list(vnic_indices)
+
+        return all_vnic_indices
+
+    def update_nat_rules(self, context, router, router_id):
+        router_ids = self.edge_manager.get_routers_on_same_edge(
+            context, router_id)
+        self._update_nat_rules_on_routers(context, router_id, router_ids)
+
     def _update_nat_rules_on_routers(self, context,
                                      target_router_id, router_ids):
         snats = []
         dnats = []
+        vnics_by_router = self._get_all_routers_vnic_indices(
+            context, router_ids)
         for router_id in router_ids:
             router_qry = context.session.query(l3_db.Router)
             router = router_qry.filter_by(id=router_id).one()
@@ -110,6 +200,21 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                 snat, dnat = self.plugin._get_nat_rules(context, router)
                 snats.extend(snat)
                 dnats.extend(dnat)
+                if len(dnat) > 0:
+                    # Copy each DNAT rule to all vnics of the other routers,
+                    # to allow NAT-ed traffic between routers
+                    other_vnics = []
+                    for other_router_id in router_ids:
+                        if other_router_id != router_id:
+                            other_vnics.extend(
+                                vnics_by_router[other_router_id])
+                    for rule in dnat:
+                        for vnic_index in other_vnics:
+                            new_rule = rule.copy()
+                            # use explicit vnic_index
+                            new_rule['vnic_index'] = vnic_index
+                            dnats.extend([new_rule])
+
         edge_utils.update_nat_rules(
             self.nsx_v, context, target_router_id, snats, dnats)
 
@@ -212,9 +317,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                     context, router_id, router_db.admin_state_up)
                 new_edge_id = edge_utils.get_router_edge_id(context,
                                                             router_id)
-                with locking.LockManager.get_lock(
-                        str(new_edge_id),
-                        lock_file_prefix=NSXV_ROUTER_RECONFIG):
+                with locking.LockManager.get_lock(str(new_edge_id)):
                     self._add_router_services_on_available_edge(context,
                                                                 router_id)
             else:
@@ -367,18 +470,6 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
         intf_num = len(router_net_ids)
         return (conflict_network_ids, conflict_router_ids, intf_num)
 
-    def _get_external_network_id_by_router(self, context, router_id):
-        """Get router's external network id if it has."""
-        router = self.plugin.get_router(context, router_id)
-        ports_qry = context.session.query(models_v2.Port)
-        gw_ports = ports_qry.filter_by(
-            device_id=router_id,
-            device_owner=l3_db.DEVICE_OWNER_ROUTER_GW,
-            id=router['gw_port_id']).all()
-
-        if gw_ports:
-            return gw_ports[0]['network_id']
-
     def _get_conflict_network_ids_by_ext_net(self, context, router_id):
         """Collect conflicting networks based on external network.
         Collect conflicting networks which has overlapping subnet with the
@@ -455,7 +546,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
         return optional_router_ids, conflict_router_ids
 
     def _bind_router_on_available_edge(self, context, router_id, admin_state):
-        with locking.LockManager.get_lock("router", lock_file_prefix="bind-"):
+        with locking.LockManager.get_lock('nsx-shared-router-pool'):
             conflict_network_ids, conflict_router_ids, intf_num = (
                 self._get_conflict_network_and_router_ids_by_intf(context,
                                                                   router_id))
@@ -475,9 +566,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
             if metadata_proxy_handler and new:
                 metadata_proxy_handler.configure_router_edge(router_id)
             edge_id = edge_utils.get_router_edge_id(context, router_id)
-            with locking.LockManager.get_lock(
-                    str(edge_id),
-                    lock_file_prefix=NSXV_ROUTER_RECONFIG):
+            with locking.LockManager.get_lock(str(edge_id)):
                 # add all internal interfaces of the router on edge
                 intf_net_ids = (
                     self.plugin._get_internal_network_ids_by_router(context,
@@ -500,7 +589,8 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
             context, router_id)
         self._update_external_interface_on_routers(
             context, router_id, router_ids)
-        self._update_routes_on_routers(context, router_id, router_ids)
+        self._update_routes_on_routers(context, router_id, router_ids,
+                                       only_if_target_routes=True)
         self._update_nat_rules_on_routers(context, router_id, router_ids)
         self._update_subnets_and_dnat_firewall_on_routers(
             context, router_id, router_ids, allow_external=True)
@@ -511,7 +601,8 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
             context, router_id)
         router_ids.remove(router_id)
         # Refresh firewall, nats, ext_vnic as well as static routes
-        self._update_routes_on_routers(context, router_id, router_ids)
+        self._update_routes_on_routers(context, router_id, router_ids,
+                                       only_if_target_routes=True)
         self._update_subnets_and_dnat_firewall_on_routers(
             context, router_id, router_ids, allow_external=True)
         self._update_nat_rules_on_routers(context, router_id, router_ids)
@@ -527,7 +618,8 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                                         is_wait=False)
 
     def _update_router_gw_info(self, context, router_id, info,
-                               is_routes_update=False):
+                               is_routes_update=False,
+                               force_update=False):
         router = self.plugin._get_router(context, router_id)
         edge_id = edge_utils.get_router_edge_id(context, router_id)
         if not edge_id:
@@ -536,9 +628,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
         # UPDATE gw info only if the router has been attached to an edge
         else:
             is_migrated = False
-            with locking.LockManager.get_lock(
-                    str(edge_id),
-                    lock_file_prefix=NSXV_ROUTER_RECONFIG):
+            with locking.LockManager.get_lock(str(edge_id)):
                 router_ids = self.edge_manager.get_routers_on_same_edge(
                     context, router_id)
                 org_ext_net_id = (router.gw_port_id and
@@ -579,14 +669,6 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                         self._unbind_router_on_edge(context, router_id)
                         is_migrated = True
                     else:
-                        # Clear gateway info if all routers has no gw conf
-                        if (orgnexthop and
-                            (org_ext_net_id != new_ext_net_id or
-                             len(ext_net_ids) == 0)):
-                            LOG.debug("Delete default gateway %s", orgnexthop)
-                            edge_utils.clear_gateway(self.nsx_v, context,
-                                                     router_id)
-
                         # Update external vnic if addr or mask is changed
                         if orgaddr != newaddr or orgmask != newmask:
                             self._update_external_interface_on_routers(
@@ -602,8 +684,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                                                               router_ids)
 
                         if (new_ext_net_id != org_ext_net_id or
-                            new_enable_snat != org_enable_snat or
-                            is_routes_update):
+                            new_enable_snat != org_enable_snat):
                             self._update_subnets_and_dnat_firewall_on_routers(
                                 context, router_id, router_ids,
                                 allow_external=True)
@@ -615,102 +696,102 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                 self._bind_router_on_available_edge(
                     context, router_id, router.admin_state_up)
                 edge_id = edge_utils.get_router_edge_id(context, router_id)
-                with locking.LockManager.get_lock(
-                        str(edge_id),
-                        lock_file_prefix=NSXV_ROUTER_RECONFIG):
+                with locking.LockManager.get_lock(str(edge_id)):
                     self._add_router_services_on_available_edge(context,
                                                                 router_id)
 
+    def _base_add_router_interface(self, context, router_id, interface_info):
+        with locking.LockManager.get_lock('nsx-shared-router-pool'):
+            return super(nsx_v.NsxVPluginV2, self.plugin).add_router_interface(
+                context, router_id, interface_info)
+
     def add_router_interface(self, context, router_id, interface_info):
         self.plugin._check_intf_number_of_router(context, router_id)
         edge_id = edge_utils.get_router_edge_id(context, router_id)
         router_db = self.plugin._get_router(context, router_id)
         if edge_id:
             is_migrated = False
-            with locking.LockManager.get_lock(
-                    str(edge_id),
-                    lock_file_prefix=NSXV_ROUTER_RECONFIG):
-                router_ids = self.edge_manager.get_routers_on_same_edge(
-                    context, router_id)
-                info = super(nsx_v.NsxVPluginV2,
-                       self.plugin).add_router_interface(
-                    context, router_id, interface_info)
-                subnet = self.plugin.get_subnet(context, info['subnet_id'])
-                network_id = subnet['network_id']
-                # Collect all conflict networks whose cidr are overlapped
-                # with networks attached to the router and conflct routers
-                # which has same network with the router's.
-                conflict_network_ids, conflict_router_ids, _ = (
-                    self._get_conflict_network_and_router_ids_by_intf(
-                        context, router_id))
+            with locking.LockManager.get_lock('nsx-shared-router-pool'):
+                with locking.LockManager.get_lock(str(edge_id)):
+                    router_ids = self.edge_manager.get_routers_on_same_edge(
+                        context, router_id)
+                    info = super(nsx_v.NsxVPluginV2,
+                                 self.plugin).add_router_interface(
+                        context, router_id, interface_info)
+                    subnet = self.plugin.get_subnet(context, info['subnet_id'])
+                    network_id = subnet['network_id']
+                    # Collect all conflict networks whose cidr are overlapped
+                    # with networks attached to the router and conflict routers
+                    # which has same network with the router's.
+                    conflict_network_ids, conflict_router_ids, _ = (
+                        self._get_conflict_network_and_router_ids_by_intf(
+                            context, router_id))
 
-                _, new_conflict_router_ids = (
-                    self._get_available_and_conflicting_ids(context,
-                                                            router_id))
-                conflict_router_ids.extend(new_conflict_router_ids)
-                conflict_router_ids = list(set(conflict_router_ids))
+                    _, new_conflict_router_ids = (
+                        self._get_available_and_conflicting_ids(context,
+                                                                router_id))
+                    conflict_router_ids.extend(new_conflict_router_ids)
+                    conflict_router_ids = list(set(conflict_router_ids))
 
-                interface_ports = (
-                    self.plugin._get_router_interface_ports_by_network(
-                        context, router_id, network_id))
-                # Consider whether another subnet of the same network
-                # has been attached to the router.
-                if len(interface_ports) > 1:
-                    is_conflict = self.edge_manager.is_router_conflict_on_edge(
-                        context, router_id, conflict_router_ids,
-                        conflict_network_ids, 0)
-                else:
-                    is_conflict = self.edge_manager.is_router_conflict_on_edge(
-                        context, router_id, conflict_router_ids,
-                        conflict_network_ids, 1)
-                if is_conflict:
+                    interface_ports = (
+                        self.plugin._get_router_interface_ports_by_network(
+                            context, router_id, network_id))
+                    # Consider whether another subnet of the same network
+                    # has been attached to the router.
                     if len(interface_ports) > 1:
-                        self._remove_router_services_on_edge(
-                            context, router_id)
+                        is_conflict = (
+                            self.edge_manager.is_router_conflict_on_edge(
+                                context, router_id, conflict_router_ids,
+                                conflict_network_ids, 0))
                     else:
-                        self._remove_router_services_on_edge(
+                        is_conflict = (
+                            self.edge_manager.is_router_conflict_on_edge(
+                                context, router_id, conflict_router_ids,
+                                conflict_network_ids, 1))
+                    if is_conflict:
+                        if len(interface_ports) > 1:
+                            self._remove_router_services_on_edge(
+                                context, router_id)
+                        else:
+                            self._remove_router_services_on_edge(
+                                context, router_id, network_id)
+                        self._unbind_router_on_edge(context, router_id)
+                        is_migrated = True
+                    else:
+                        address_groups = self.plugin._get_address_groups(
                             context, router_id, network_id)
-                    self._unbind_router_on_edge(context, router_id)
-                    is_migrated = True
-                else:
-                    address_groups = self.plugin._get_address_groups(
-                        context, router_id, network_id)
-                    edge_utils.update_internal_interface(
-                        self.nsx_v, context, router_id,
-                        network_id, address_groups, router_db.admin_state_up)
-                    if router_db.gw_port and router_db.enable_snat:
-                        self._update_nat_rules_on_routers(
-                            context, router_id, router_ids)
-                    self._update_subnets_and_dnat_firewall_on_routers(
-                        context, router_id, router_ids, allow_external=True)
+                        edge_utils.update_internal_interface(
+                            self.nsx_v, context, router_id,
+                            network_id, address_groups,
+                            router_db.admin_state_up)
+                        if router_db.gw_port and router_db.enable_snat:
+                            self._update_nat_rules_on_routers(
+                                context, router_id, router_ids)
+                        self._update_subnets_and_dnat_firewall_on_routers(
+                            context, router_id, router_ids,
+                            allow_external=True)
             if is_migrated:
                 self._bind_router_on_available_edge(
                     context, router_id, router_db.admin_state_up)
                 edge_id = edge_utils.get_router_edge_id(context, router_id)
-                with locking.LockManager.get_lock(
-                        str(edge_id),
-                        lock_file_prefix=NSXV_ROUTER_RECONFIG):
+                with locking.LockManager.get_lock(str(edge_id)):
                     self._add_router_services_on_available_edge(context,
                                                                 router_id)
         else:
-            info = super(nsx_v.NsxVPluginV2, self.plugin).add_router_interface(
-                context, router_id, interface_info)
-            # bind and configure routing servie on an availab edge
+            info = self._base_add_router_interface(context, router_id,
+                                                   interface_info)
+            # bind and configure routing service on an available edge
             self._bind_router_on_available_edge(
                 context, router_id, router_db.admin_state_up)
             edge_id = edge_utils.get_router_edge_id(context, router_id)
-            with locking.LockManager.get_lock(
-                    str(edge_id),
-                    lock_file_prefix=NSXV_ROUTER_RECONFIG):
+            with locking.LockManager.get_lock(str(edge_id)):
                 self._add_router_services_on_available_edge(context,
                                                             router_id)
         return info
 
     def remove_router_interface(self, context, router_id, interface_info):
         edge_id = edge_utils.get_router_edge_id(context, router_id)
-        with locking.LockManager.get_lock(
-                str(edge_id),
-                lock_file_prefix=NSXV_ROUTER_RECONFIG):
+        with locking.LockManager.get_lock(str(edge_id)):
             info = super(
                 nsx_v.NsxVPluginV2, self.plugin).remove_router_interface(
                     context, router_id, interface_info)
@@ -740,9 +821,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
 
     def _update_edge_router(self, context, router_id):
         edge_id = edge_utils.get_router_edge_id(context, router_id)
-        with locking.LockManager.get_lock(
-                str(edge_id),
-                lock_file_prefix=NSXV_ROUTER_RECONFIG):
+        with locking.LockManager.get_lock(str(edge_id)):
             router_ids = self.edge_manager.get_routers_on_same_edge(
                 context, router_id)
             if router_ids:
