@@ -21,7 +21,7 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 
-from neutron.i18n import _LE, _LI, _LW
+from vmware_nsx._i18n import _LE, _LI, _LW
 from vmware_nsx.common import nsxv_constants
 from vmware_nsx.common import utils
 from vmware_nsx.plugins.nsx_v.vshield.common import constants
@@ -51,6 +51,16 @@ class EdgeApplianceDriver(object):
             'fqdn': None,
             'enableAesni': enable_aesni,
             'enableFips': enable_fips,
+            'featureConfigs': {
+                'features': [
+                    {
+                        'featureType': 'firewall_4.0',
+                        'globalConfig': {
+                            'tcpTimeoutEstablished': 7200
+                        }
+                    }
+                ]
+            },
             'cliSettings': {
                 'remoteAccess': remote_access
             },
@@ -75,6 +85,9 @@ class EdgeApplianceDriver(object):
         if datacenter_moid:
             edge['datacenterMoid'] = datacenter_moid
 
+        if not dist and cfg.CONF.nsxv.edge_ha:
+            self._enable_high_availability(edge)
+
         return edge
 
     def _assemble_edge_appliance(self, resource_pool_id, datastore_id):
@@ -473,7 +486,7 @@ class EdgeApplianceDriver(object):
         edge = self._assemble_edge(
             edge_name, datacenter_moid=self.datacenter_moid,
             deployment_container_id=self.deployment_container_id,
-            appliance_size=appliance_size, remote_access=True, dist=dist)
+            appliance_size=appliance_size, remote_access=False, dist=dist)
         appliance = self._assemble_edge_appliance(self.resource_pool_id,
                                                   self.datastore_id)
         if appliance:
@@ -496,19 +509,17 @@ class EdgeApplianceDriver(object):
                 constants.INTEGRATION_SUBNET_NETMASK,
                 type="internal")
             edge['vnics']['vnics'].append(vnic_inside)
+
         # If default login credentials for Edge are set, configure accordingly
         if (cfg.CONF.nsxv.edge_appliance_user and
             cfg.CONF.nsxv.edge_appliance_password):
-            edge['cliSettings'] = {
+            edge['cliSettings'].update({
                 'userName': cfg.CONF.nsxv.edge_appliance_user,
-                'password': cfg.CONF.nsxv.edge_appliance_password}
+                'password': cfg.CONF.nsxv.edge_appliance_password})
 
         if not dist and loadbalancer_enable:
             self._enable_loadbalancer(edge)
 
-        if not dist and cfg.CONF.nsxv.edge_ha:
-            self._enable_high_availability(edge)
-
         if async:
             userdata = {
                 'dist': dist,
@@ -556,14 +567,15 @@ class EdgeApplianceDriver(object):
 
     def update_edge(self, router_id, edge_id, name, internal_network,
                     jobdata=None, dist=False, loadbalancer_enable=True,
-                    appliance_size=nsxv_constants.LARGE):
+                    appliance_size=nsxv_constants.LARGE,
+                    set_errors=False):
         """Update edge name."""
         task_name = 'update-%s' % name
         edge_name = name
         edge = self._assemble_edge(
             edge_name, datacenter_moid=self.datacenter_moid,
             deployment_container_id=self.deployment_container_id,
-            appliance_size=appliance_size, remote_access=True, dist=dist)
+            appliance_size=appliance_size, remote_access=False, dist=dist)
         edge['id'] = edge_id
         appliance = self._assemble_edge_appliance(self.resource_pool_id,
                                                   self.datastore_id)
@@ -591,9 +603,11 @@ class EdgeApplianceDriver(object):
         if not dist and loadbalancer_enable:
             self._enable_loadbalancer(edge)
         userdata = {
+            'router_id': router_id,
             'edge_id': edge_id,
             'request': edge,
-            'jobdata': jobdata
+            'jobdata': jobdata,
+            'set_errors': set_errors
         }
         task = tasks.Task(task_name, router_id,
                           self._update_edge,
@@ -803,14 +817,23 @@ class EdgeApplianceDriver(object):
         nat_rules = []
 
         for dnat in dnats:
+            vnic_index = constants.EXTERNAL_VNIC_INDEX
+            if 'vnic_index' in dnat:
+                vnic_index = dnat['vnic_index']
             nat_rules.append(self._assemble_nat_rule(
-                'dnat', dnat['dst'], dnat['translated']))
+                'dnat', dnat['dst'], dnat['translated'], vnic_index=vnic_index
+            ))
             nat_rules.append(self._assemble_nat_rule(
-                'snat', dnat['translated'], dnat['dst']))
+                'snat', dnat['translated'], dnat['dst'], vnic_index=vnic_index
+            ))
 
         for snat in snats:
+            vnic_index = constants.EXTERNAL_VNIC_INDEX
+            if 'vnic_index' in snat:
+                vnic_index = snat['vnic_index']
             nat_rules.append(self._assemble_nat_rule(
-                'snat', snat['src'], snat['translated']))
+                'snat', snat['src'], snat['translated'], vnic_index=vnic_index
+            ))
 
         userdata = {
             'edge_id': edge_id,
@@ -856,7 +879,6 @@ class EdgeApplianceDriver(object):
             # for speedup
             return task_constants.TaskStatus.ABORT
         gateway = task.userdata['gateway']
-        gateway_vnic_index = task.userdata['gateway_vnic_index']
         routes = task.userdata['routes']
         LOG.debug("VCNS: start updating routes for %s", edge_id)
         static_routes = []
@@ -883,8 +905,7 @@ class EdgeApplianceDriver(object):
         if gateway:
             request["defaultRoute"] = {
                 "description": "default-gateway",
-                "gatewayAddress": gateway,
-                "vnic": gateway_vnic_index
+                "gatewayAddress": gateway
             }
         try:
             self.vcns.update_routes(edge_id, request)
