@@ -16,7 +16,7 @@ from neutron.db import db_base_plugin_v2
 from oslo_log import log as logging
 from oslo_utils import excutils
 
-from neutron.i18n import _, _LE
+from vmware_nsx._i18n import _, _LE
 from vmware_nsx.db import nsxv_db
 from vmware_nsx.plugins.nsx_v.vshield.common import (
     exceptions as vcns_exc)
@@ -64,13 +64,31 @@ class EdgeFirewallDriver(db_base_plugin_v2.NeutronDbPluginV2):
         else:
             return '%d:%d' % (min_port, max_port)
 
-    def _get_min_max_ports_from_range(self, port_range):
-        if not port_range:
-            return [None, None]
-        min_port, sep, max_port = port_range.partition(":")
-        if not max_port:
-            max_port = min_port
-        return [int(min_port), int(max_port)]
+    def _get_ports_list_from_string(self, port_str):
+        """Receives a string representation of the service ports,
+        and return a list of integers
+        Supported formats:
+        Empty string - no ports
+        "number" - a single port
+        "num1:num2" - a range
+        "num1,num2,num3" - a list
+        """
+        if not port_str:
+            return []
+        if ':' in port_str:
+            min_port, sep, max_port = port_str.partition(":")
+            return list(range(int(min_port.strip()),
+                              int(max_port.strip()) + 1))
+        if ',' in port_str:
+            # remove duplications (using set) and empty/non numeric entries
+            ports_set = set()
+            for orig_port in port_str.split(','):
+                port = orig_port.strip()
+                if port and port.isdigit():
+                    ports_set.add(int(port))
+            return sorted(list(ports_set))
+        else:
+            return [int(port_str.strip())]
 
     def _convert_firewall_rule(self, context, rule, index=None):
         vcns_rule = {
@@ -100,13 +118,11 @@ class EdgeFirewallDriver(db_base_plugin_v2.NeutronDbPluginV2):
             vcns_rule['application'] = rule['application']
         service = {}
         if rule.get('source_port'):
-            min_port, max_port = self._get_min_max_ports_from_range(
+            service['sourcePort'] = self._get_ports_list_from_string(
                 rule['source_port'])
-            service['sourcePort'] = [i for i in range(min_port, max_port + 1)]
         if rule.get('destination_port'):
-            min_port, max_port = self._get_min_max_ports_from_range(
+            service['port'] = self._get_ports_list_from_string(
                 rule['destination_port'])
-            service['port'] = [i for i in range(min_port, max_port + 1)]
         if rule.get('protocol'):
             service['protocol'] = rule['protocol']
             if rule['protocol'] == 'icmp':
@@ -114,6 +130,8 @@ class EdgeFirewallDriver(db_base_plugin_v2.NeutronDbPluginV2):
                     service['icmpType'] = rule['icmp_type']
                 else:
                     service['icmpType'] = 'any'
+        if rule.get('ruleId'):
+            vcns_rule['ruleId'] = rule.get('ruleId')
         if service:
             vcns_rule['application'] = {
                 'service': [service]
@@ -163,6 +181,7 @@ class EdgeFirewallDriver(db_base_plugin_v2.NeutronDbPluginV2):
                  'ruleTag': ruleTag})
         return {
             'featureType': "firewall_4.0",
+            'globalConfig': {'tcpTimeoutEstablished': 7200},
             'firewallRules': {
                 'firewallRules': vcns_rules}}
 
