@@ -14,6 +14,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
+
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
@@ -21,6 +23,8 @@ import retrying
 import six
 import xml.etree.ElementTree as et
 
+from vmware_nsx._i18n import _LE
+from vmware_nsx.common import nsxv_constants
 from vmware_nsx.plugins.nsx_v.vshield.common import exceptions
 from vmware_nsx.plugins.nsx_v.vshield.common import VcnsApiClient
 
@@ -42,6 +46,7 @@ SECURITYGROUP_PREFIX = '/api/2.0/services/securitygroup'
 VDN_PREFIX = '/api/2.0/vdn'
 SERVICES_PREFIX = '/api/2.0/services'
 SPOOFGUARD_PREFIX = '/api/4.0/services/spoofguard'
+TRUSTSTORE_PREFIX = '%s/%s' % (SERVICES_PREFIX, 'truststore')
 
 #LbaaS Constants
 LOADBALANCER_SERVICE = "loadbalancer/config"
@@ -65,8 +70,12 @@ SYSCTL_SERVICE = 'systemcontrol/config'
 # L2 gateway constants
 BRIDGE = "bridging/config"
 
+# Self Signed Certificate constants
+CSR = "csr"
+CERTIFICATE = "certificate"
 
-def retry_upon_exception(exc, delay=500, max_delay=2000,
+
+def retry_upon_exception(exc, delay=500, max_delay=4000,
                          max_attempts=cfg.CONF.nsxv.retries):
     return retrying.retry(retry_on_exception=lambda e: isinstance(e, exc),
                           wait_exponential_multiplier=delay,
@@ -109,8 +118,15 @@ class Vcns(object):
             _client = self.jsonapi_client.request
         else:
             _client = self.xmlapi_client.request
+
+        ts = time.time()
         header, content = self._client_request(_client, method, uri, params,
                                                headers, encodeParams)
+        te = time.time()
+
+        LOG.debug('VcnsApiHelper reply: header=%(header)s content=%(content)s'
+                  ' took %(seconds)2.4f',
+                  {'header': header, 'content': content, 'seconds': te - ts})
         if content == '':
             return header, {}
         if kwargs.get('decode', True):
@@ -121,14 +137,18 @@ class Vcns(object):
         uri = URI_PREFIX + "?lockUpdatesOnEdge=true"
         return self.do_request(HTTP_POST, uri, decode=False)
 
+    @retry_upon_exception(exceptions.ResourceNotFound)
+    @retry_upon_exception(exceptions.RequestBad)
     def deploy_edge(self, request, async=True):
         uri = URI_PREFIX
         if async:
             uri += "?async=true"
         return self.do_request(HTTP_POST, uri, request, decode=False)
 
-    def update_edge(self, edge_id, request):
-        uri = "%s/%s?async=true" % (URI_PREFIX, edge_id)
+    def update_edge(self, edge_id, request, async=False):
+        uri = "%s/%s" % (URI_PREFIX, edge_id)
+        if async:
+            uri += "?async=true"
         return self.do_request(HTTP_PUT, uri, request, decode=False)
 
     def get_edge_id(self, job_id):
@@ -151,12 +171,17 @@ class Vcns(object):
         uri = "%s/%s/interfaces?action=patch" % (URI_PREFIX, edge_id)
         return self.do_request(HTTP_POST, uri, interface, decode=True)
 
+    def get_vdr_internal_interface(self, edge_id, interface_index):
+        uri = "%s/%s/interfaces/%s" % (URI_PREFIX, edge_id, interface_index)
+        return self.do_request(HTTP_GET, uri, decode=True)
+
     def update_vdr_internal_interface(self, edge_id,
                                       interface_index, interface):
         uri = "%s/%s/interfaces/%s" % (URI_PREFIX, edge_id, interface_index)
         return self.do_request(HTTP_PUT, uri, interface,
                                format='xml', decode=True)
 
+    @retry_upon_exception(exceptions.RequestBad)
     def delete_vdr_internal_interface(self, edge_id, interface_index):
         uri = "%s/%s/interfaces/%d" % (URI_PREFIX, edge_id, interface_index)
         return self.do_request(HTTP_DELETE, uri, decode=True)
@@ -190,6 +215,10 @@ class Vcns(object):
         uri = "%s/%s/status?getlatest=false" % (URI_PREFIX, edge_id)
         return self.do_request(HTTP_GET, uri, decode=True)
 
+    def get_edge(self, edge_id):
+        uri = "%s/%s" % (URI_PREFIX, edge_id)
+        return self.do_request(HTTP_GET, uri, decode=True)
+
     def get_edges(self):
         uri = URI_PREFIX
         return self.do_request(HTTP_GET, uri, decode=True)
@@ -203,7 +232,7 @@ class Vcns(object):
         return self.do_request(HTTP_GET, uri)
 
     def update_routes(self, edge_id, routes):
-        uri = "%s/%s/routing/config/static?async=true" % (URI_PREFIX, edge_id)
+        uri = "%s/%s/routing/config/static" % (URI_PREFIX, edge_id)
         return self.do_request(HTTP_PUT, uri, routes)
 
     def create_lswitch(self, lsconfig):
@@ -432,13 +461,21 @@ class Vcns(object):
                                                         portgroup_id)
         return self.do_request(HTTP_DELETE, uri, format='xml', decode=False)
 
+    def get_vdn_switch(self, dvs_id):
+        uri = '/api/2.0/vdn/switches/%s' % dvs_id
+        return self.do_request(HTTP_GET, uri, decode=True)
+
+    def update_vdn_switch(self, switch):
+        uri = '/api/2.0/vdn/switches'
+        return self.do_request(HTTP_PUT, uri, switch)
+
     def query_interface(self, edge_id, vnic_index):
         uri = "%s/%s/vnics/%d" % (URI_PREFIX, edge_id, vnic_index)
         return self.do_request(HTTP_GET, uri, decode=True)
 
     def reconfigure_dhcp_service(self, edge_id, request_config):
         """Reconfigure dhcp static bindings in the created Edge."""
-        uri = "/api/4.0/edges/%s/dhcp/config?async=true" % edge_id
+        uri = "/api/4.0/edges/%s/dhcp/config" % edge_id
 
         return self.do_request(HTTP_PUT, uri, request_config)
 
@@ -484,11 +521,14 @@ class Vcns(object):
         return self.do_request(HTTP_PUT, uri, et.tostring(sg),
                                format='xml', decode=False, encode=False)
 
+    def list_security_groups(self):
+        uri = '%s/scope/globalroot-0' % SECURITYGROUP_PREFIX
+        return self.do_request(HTTP_GET, uri, format='xml', decode=False)
+
     def get_security_group_id(self, sg_name):
         """Returns NSXv security group id which match the given name."""
-        uri = '%s/scope/globalroot-0' % SECURITYGROUP_PREFIX
-        h, c = self.do_request(HTTP_GET, uri, format='xml', decode=False)
-        root = et.fromstring(c)
+        h, secgroups = self.list_security_groups()
+        root = et.fromstring(secgroups)
         for sg in root.iter('securitygroup'):
             if sg.find('name').text == sg_name:
                 return sg.find('objectId').text
@@ -506,7 +546,7 @@ class Vcns(object):
         uri = self._build_uri_path(edge_id, BRIDGE)
         return self.do_request(HTTP_DELETE, uri, format='xml', decode=False)
 
-    def create_section(self, type, request):
+    def create_section(self, type, request, insert_before=None):
         """Creates a layer 3 or layer 2 section in nsx rule table.
 
         The method will return the uri to newly created section.
@@ -516,6 +556,13 @@ class Vcns(object):
         else:
             sec_type = 'layer2sections'
         uri = '%s/%s?autoSaveDraft=false' % (FIREWALL_PREFIX, sec_type)
+        # We want to place security-group sections before the default cluster
+        # section, and we want to place the default cluster section before the
+        # global default section.
+        if insert_before:
+            uri += '&operation=insert_before&anchorId=%s' % insert_before
+        else:
+            uri += '&operation=insert_before&anchorId=1003'
         return self.do_request(HTTP_POST, uri, request, format='xml',
                                decode=False, encode=False)
 
@@ -535,14 +582,14 @@ class Vcns(object):
         return self.do_request(HTTP_GET, section_uri, format='xml',
                                decode=False)
 
+    def get_dfw_config(self):
+        uri = FIREWALL_PREFIX
+        return self.do_request(HTTP_GET, uri, decode=False, format='xml')
+
     def get_section_id(self, section_name):
         """Retrieve the id of a section from nsx."""
-        uri = FIREWALL_PREFIX
-        h, section_list = self.do_request(HTTP_GET, uri, decode=False,
-                                          format='xml')
-
-        root = et.fromstring(section_list)
-
+        h, firewall_config = self.get_dfw_config()
+        root = et.fromstring(firewall_config)
         for sec in root.iter('section'):
             if sec.attrib['name'] == section_name:
                 return sec.attrib['id']
@@ -637,6 +684,14 @@ class Vcns(object):
         uri = '%s/policies/%s' % (SPOOFGUARD_PREFIX, policy_id)
         return self.do_request(HTTP_DELETE, uri, decode=False)
 
+    def get_spoofguard_policy(self, policy_id):
+        uri = '%s/policies/%s' % (SPOOFGUARD_PREFIX, policy_id)
+        return self.do_request(HTTP_GET, uri, decode=True)
+
+    def get_spoofguard_policies(self):
+        uri = '%s/policies/' % SPOOFGUARD_PREFIX
+        return self.do_request(HTTP_GET, uri, decode=True)
+
     def _approve_assigned_addresses(self, policy_id,
                                     vnic_id, mac_addr, addresses):
         uri = '%s/%s' % (SPOOFGUARD_PREFIX, policy_id)
@@ -696,6 +751,12 @@ class Vcns(object):
         else:
             return uri_path
 
+    def get_scoping_objects(self):
+        uri = '%s/usermgmt/scopingobjects' % SERVICES_PREFIX
+        h, scoping_objects = self.do_request(HTTP_GET, uri, decode=False,
+                                             format='xml')
+        return scoping_objects
+
     def _scopingobjects_lookup(self, type_name, object_id, name=None):
         uri = '%s/usermgmt/scopingobjects' % SERVICES_PREFIX
         h, so_list = self.do_request(HTTP_GET, uri, decode=False,
@@ -757,16 +818,22 @@ class Vcns(object):
             return False
         return True
 
-    def get_version(self):
-        uri = '/api/1.0/appliance-management/summary/system'
+    def _get_version(self):
+        uri = '/api/2.0/services/vsmconfig'
         h, c = self.do_request(HTTP_GET, uri, decode=True)
-        version = '%s.%s.%s' % (c['versionInfo']['majorVersion'],
-                                c['versionInfo']['minorVersion'],
-                                c['versionInfo']['patchVersion'])
-        LOG.debug("NSX Version: %s, Build: %s",
-                  version, c['versionInfo']['buildNumber'])
+        version = c['version']
+        LOG.debug("NSX Version: %s", version)
         return version
 
+    def get_version(self):
+        try:
+            return self._get_version()
+        except Exception as e:
+            # Versions prior to 6.2.0 do not support the above API
+            LOG.error(_LE("Unable to get NSX version. Exception: %s"), e)
+            # Minimum supported version is 6.1
+            return '6.1'
+
     def get_tuning_configration(self):
         uri = '/api/4.0/edgePublish/tuningConfiguration'
         h, c = self.do_request(HTTP_GET, uri, decode=True)
@@ -788,7 +855,32 @@ class Vcns(object):
         return self.do_request(HTTP_PUT, uri, et.tostring(tuning),
                                format='xml', decode=True)
 
-    def enable_ha(self, edge_id, request_config):
+    def enable_ha(self, edge_id, request_config, async=True):
         """Enable HA in the given edge."""
-        uri = "/api/4.0/edges/%s/highavailability/config?async=true" % edge_id
+        uri = "/api/4.0/edges/%s/highavailability/config" % edge_id
+        if async:
+            uri += "?async=true"
         return self.do_request(HTTP_PUT, uri, request_config)
+
+    def change_edge_appliance_size(self, edge_id, size):
+        """Change the size of edge appliances."""
+        uri = ("/api/4.0/edges/%s/appliances/?size=%s" %
+               (edge_id, size))
+        return self.do_request(HTTP_POST, uri)
+
+    def upload_edge_certificate(self, edge_id, request):
+        """Creates a certificate on the specified Edge appliance."""
+        uri = '%s/%s/%s' % (TRUSTSTORE_PREFIX, CERTIFICATE, edge_id)
+        return self.do_request(HTTP_POST, uri, request, decode=True)
+
+    def create_csr(self, edge_id, request=nsxv_constants.CSR_REQUEST):
+        """Create a CSR on the specified Edge appliance."""
+        uri = '%s/%s/%s' % (TRUSTSTORE_PREFIX, CSR, edge_id)
+        return self.do_request(HTTP_POST, uri, request, format='xml',
+                               decode=False)
+
+    def create_csr_cert(self, csr_id):
+        """Create a CSR self signed cert on the specified Edge appliance."""
+        uri = '%s/%s/%s?noOfDays=%s' % (TRUSTSTORE_PREFIX, CSR, csr_id,
+                                        nsxv_constants.CERT_NUMBER_OF_DAYS)
+        return self.do_request(HTTP_PUT, uri)
