@@ -14,10 +14,10 @@
 # limitations under the License.
 
 import mock
+from neutron_lib import constants as n_consts
+from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 
-from neutron.common import constants as n_consts
-from neutron.common import exceptions as n_exc
 from neutron import context
 from neutron.tests import base
 from neutron.tests.unit import testlib_api
@@ -354,7 +354,7 @@ class LsnManagerTestCase(base.BaseTestCase):
 
     def _test_lsn_delete_by_network_with_exc(self, exc):
         self.mock_lsn_api.lsn_for_network_get.side_effect = exc
-        with mock.patch.object(lsn_man.LOG, 'warn') as l:
+        with mock.patch.object(lsn_man.LOG, 'warning') as l:
             self.manager.lsn_delete_by_network(mock.ANY, self.net_id)
             self.assertEqual(1, l.call_count)
 
@@ -428,7 +428,7 @@ class LsnManagerTestCase(base.BaseTestCase):
 
     def _test_lsn_port_delete_with_exc(self, exc):
         self.mock_lsn_api.lsn_port_delete.side_effect = exc
-        with mock.patch.object(lsn_man.LOG, 'warn') as l:
+        with mock.patch.object(lsn_man.LOG, 'warning') as l:
             self.manager.lsn_port_delete(mock.ANY, mock.ANY, mock.ANY)
             self.assertEqual(1, self.mock_lsn_api.lsn_port_delete.call_count)
             self.assertEqual(1, l.call_count)
@@ -656,7 +656,7 @@ class LsnManagerTestCase(base.BaseTestCase):
     def test_lsn_port_dispose_api_error(self):
         self.mock_lsn_api.lsn_port_delete.side_effect = (
             exception.NsxApiException)
-        with mock.patch.object(lsn_man.LOG, 'warn') as l:
+        with mock.patch.object(lsn_man.LOG, 'warning') as l:
             self.manager.lsn_port_dispose(mock.ANY, self.net_id, self.mac)
             self.assertEqual(1, l.call_count)
 
