@@ -19,6 +19,7 @@ from oslo_utils import uuidutils
 import six
 import xml.etree.ElementTree as ET
 
+from vmware_nsx._i18n import _
 from vmware_nsx.plugins.nsx_v.vshield.common import exceptions
 
 SECTION_LOCATION_HEADER = '/api/4.0/firewall/globalroot-0/config/%s/%s'
@@ -212,6 +213,16 @@ class FakeVcns(object):
         response = ''
         return (header, response)
 
+    def get_vdr_internal_interface(self, edge_id, interface_index):
+        response = {}
+        header = {
+            'status': 200
+        }
+        for interface in self._edges[edge_id].get('interfaces', []):
+            if int(interface['index']) == int(interface_index):
+                response = interface
+        return (header, response)
+
     def delete_vdr_internal_interface(self, edge_id, interface_index):
         for interface in self._edges[edge_id].get('interfaces', []):
             if int(interface['index']) == int(interface_index):
@@ -396,6 +407,18 @@ class FakeVcns(object):
         }
         return (header, response)
 
+    def get_edge(self, edge_id):
+        if edge_id not in self._edges:
+            raise Exception(_("Edge %s does not exist!") % edge_id)
+        header = {
+            'status': 200
+        }
+        response = {
+            'name': 'fake-edge',
+            'id': edge_id
+        }
+        return (header, response)
+
     def get_edges(self):
         header = {
             'status': 200
@@ -833,7 +856,7 @@ class FakeVcns(object):
         return (headers, response)
 
     def _unknown_error(self):
-        msg = "Unknown Error Occured.Please look into tech support logs."
+        msg = "Unknown Error Occurred.Please look into tech support logs."
         response = self._get_bad_req_response(msg, 100046, 'vShield App')
         headers = {'status': 400}
         return (headers, response)
@@ -880,7 +903,7 @@ class FakeVcns(object):
             if k not in ('ids', 'names') and v['name'] == sg_name:
                 return k
 
-    def create_section(self, type, request):
+    def create_section(self, type, request, insert_before=None):
         section = ET.fromstring(request)
         section_name = section.attrib.get('name')
         if section_name in self._sections['names']:
@@ -889,7 +912,7 @@ class FakeVcns(object):
             headers = {'status': 400}
         else:
             section_id = str(self._sections['section_ids'])
-            section.attrib['id'] = section_id
+            section.attrib['id'] = 'section-%s' % section_id
             _section = self._sections[section_id] = {'name': section_name,
                                                      'etag': 'Etag-0',
                                                      'rules': {}}
@@ -926,14 +949,13 @@ class FakeVcns(object):
             self._sections['names'].remove(_section['name'])
             _section['name'] = section_name
             self._sections['names'].add(section_name)
-            _section['rules'] = {}
             for rule in section.findall('rule'):
                 if not rule.attrib.get('id'):
                     rule.attrib['id'] = str(self._sections['rule_ids'])
                     self._sections['rule_ids'] += 1
                 rule_id = rule.attrib.get('id')
                 _section['rules'][rule_id] = ET.tostring(rule)
-            response = ET.tostring(section)
+            _, response = self._get_section(section_id)
             headers = {
                 'status': 200,
                 'location': self._get_section_location(type, section_id),
@@ -958,12 +980,14 @@ class FakeVcns(object):
         if section_id not in self._sections:
             headers, response = self._section_not_found(section_id)
         else:
-            section_rules = (''.join(self._sections[section_id]['rules'].
-                             values()))
-            response = ('<section id="%s"><rules>%s</rules></section>'
-                        % (section_id, section_rules))
-            headers = {'status': 200,
-                       'etag': self._sections[section_id]['etag']}
+            return self._get_section(section_id)
+
+    def _get_section(self, section_id):
+        section_rules = (''.join(self._sections[section_id]['rules'].values()))
+        response = ('<section id="%s">%s</section>'
+                    % (section_id, section_rules))
+        headers = {'status': 200,
+                   'etag': self._sections[section_id]['etag']}
         return (headers, response)
 
     def get_section_id(self, section_name):
@@ -1033,6 +1057,12 @@ class FakeVcns(object):
     def delete_spoofguard_policy(self, policy_id):
         self._spoofguard_policies[int(policy_id)] = {}
 
+    def get_spoofguard_policy(self, policy_id):
+        return None, self._spoofguard_policies[int(policy_id)]
+
+    def get_spoofguard_policies(self):
+        return None, self._spoofguard_policies
+
     def approve_assigned_addresses(self, policy_id,
                                    vnic_id, mac_addr, addresses):
         pass
@@ -1103,3 +1133,10 @@ class FakeVcns(object):
         }
         response = ''
         return (header, response)
+
+    def change_edge_appliance_size(self, edge_id, size):
+        header = {
+            'status': 204
+        }
+        response = {}
+        return (header, response)
