@@ -18,10 +18,11 @@ from oslo_config import cfg
 from oslo_utils import uuidutils
 from six import moves
 
-from neutron.common import exceptions as n_exc
 from neutron import context
 from neutron.plugins.common import constants as plugin_const
 from neutron.tests.unit import testlib_api
+from neutron_lib import exceptions as n_exc
+from vmware_nsx.common import exceptions as nsx_exc
 from vmware_nsx.common import nsxv_constants
 from vmware_nsx.db import nsxv_db
 from vmware_nsx.plugins.nsx_v.vshield.common import (
@@ -54,6 +55,7 @@ class EdgeUtilsTestCaseMixin(testlib_api.SqlTestCase):
         get_ver.return_value = '6.1.4'
         self.ctx = context.get_admin_context()
         self.addCleanup(nsxv_manager_p.stop)
+        self.fake_jobdata = {'router_id': 'fake_id', 'context': self.ctx}
 
     def _create_router(self, name='router1'):
         return {'name': name,
@@ -112,16 +114,76 @@ class EdgeDHCPManagerTestCase(EdgeUtilsTestCaseMixin):
                                                        fake_network['id'],
                                                        fake_subnet)
         resource_id = (vcns_const.DHCP_EDGE_PREFIX + fake_network['id'])[:36]
+        jobdata = {'router_id': resource_id, 'context': self.ctx}
         self.nsxv_manager.update_edge.assert_called_once_with(
-            resource_id, 'edge-1', mock.ANY, None,
+            resource_id, 'edge-1', mock.ANY, None, jobdata=jobdata,
             appliance_size=vcns_const.SERVICE_SIZE_MAPPING['dhcp'],
-            dist=False)
+            dist=False, set_errors=True)
+
+    def test_get_random_available_edge(self):
+        available_edge_ids = ['edge-1', 'edge-2']
+        selected_edge_id = self.edge_manager._get_random_available_edge(
+            available_edge_ids)
+        self.assertIn(selected_edge_id, available_edge_ids)
+
+    def test_get_random_available_edge_missing_edges_returns_none(self):
+        available_edge_ids = ['edge-1', 'edge-2']
+        # Always return inactive(False) while checking whether the edge
+        # exists on the backend.
+        with mock.patch.object(self.edge_manager,
+                               'check_edge_active_at_backend',
+                               return_value=False):
+            selected_edge_id = self.edge_manager._get_random_available_edge(
+                available_edge_ids)
+            # If no active edges are found on the backend, return None so that
+            # a new DHCP edge is created.
+            self.assertIsNone(selected_edge_id)
 
 
 class EdgeUtilsTestCase(EdgeUtilsTestCaseMixin):
 
+    def setUp(self):
+        super(EdgeUtilsTestCase, self).setUp()
+        self.edge_manager = edge_utils.EdgeManager(self.nsxv_manager, None)
+
+        # Args for vcns interface configuration
+        self.internal_ip = '10.0.0.1'
+        self.uplink_ip = '192.168.111.30'
+        self.subnet_mask = '255.255.255.0'
+        self.pref_len = '24'
+        self.edge_id = 'dummy'
+        self.orig_vnics = ({},
+                           {'vnics': [
+                                {'addressGroups':
+                                    {'addressGroups': [
+                                        {'subnetMask': self.subnet_mask,
+                                         'subnetPrefixLength': self.pref_len,
+                                         'primaryAddress': self.uplink_ip}]},
+                                 'type': 'uplink',
+                                 'index': 1},
+                                {'addressGroups':
+                                    {'addressGroups': [
+                                        {'subnetMask': self.subnet_mask,
+                                         'subnetPrefixLength': self.pref_len,
+                                         'primaryAddress': self.internal_ip}]},
+                                 'type': 'internal',
+                                 'index': 2}]}
+                           )
+
+        # Args for vcns vdr interface configuration
+        self.vdr_ip = '10.0.0.1'
+        self.vnic = 1
+        self.orig_vdr = ({},
+                         {'index': 2,
+                          'addressGroups': {'addressGroups':
+                                [{'subnetMask': self.subnet_mask,
+                                  'subnetPrefixLength': self.pref_len,
+                                  'primaryAddress': self.vdr_ip}]},
+                          'type': 'internal'})
+
     def test_create_lrouter(self):
         lrouter = self._create_router()
+        self.nsxv_manager.deploy_edge.reset_mock()
         edge_utils.create_lrouter(self.nsxv_manager, self.ctx, lrouter,
                                   lswitch=None, dist=False)
         self.nsxv_manager.deploy_edge.assert_called_once_with(
@@ -133,6 +195,116 @@ class EdgeUtilsTestCase(EdgeUtilsTestCaseMixin):
                      'context': self.ctx},
             appliance_size=vcns_const.SERVICE_SIZE_MAPPING['router'])
 
+    def _test_update_intereface_primary_addr(self, old_ip, new_ip, isUplink):
+        fixed_vnic = {'addressGroups':
+                      {'addressGroups': [
+                              {'subnetMask': self.subnet_mask,
+                               'subnetPrefixLength': self.pref_len,
+                               'primaryAddress': new_ip}] if new_ip else []},
+                      'type': 'uplink' if isUplink else 'internal',
+                      'index': 1 if isUplink else 2}
+
+        with mock.patch.object(self.nsxv_manager.vcns,
+            'get_interfaces', return_value=self.orig_vnics):
+            self.edge_manager.update_interface_addr(
+                self.ctx, self.edge_id, old_ip, new_ip,
+                self.subnet_mask, is_uplink=isUplink)
+            self.nsxv_manager.vcns.update_interface.assert_called_once_with(
+                self.edge_id, fixed_vnic)
+
+    def test_update_interface_addr_intrernal(self):
+        self._test_update_intereface_primary_addr(
+            self.internal_ip, '10.0.0.2', False)
+
+    def test_remove_interface_primary_addr_intrernal(self):
+        self._test_update_intereface_primary_addr(
+            self.internal_ip, None, False)
+
+    def test_update_interface_addr_uplink(self):
+        self._test_update_intereface_primary_addr(
+            self.uplink_ip, '192.168.111.31', True)
+
+    def test_remove_interface_primary_addr_uplink(self):
+        self._test_update_intereface_primary_addr(
+            self.uplink_ip, None, True)
+
+    def _test_update_intereface_secondary_addr(self, old_ip, new_ip):
+        addr_group = {'subnetMask': self.subnet_mask,
+                      'subnetPrefixLength': self.pref_len,
+                      'primaryAddress': self.uplink_ip,
+                      'secondaryAddresses': {'type': 'secondary_addresses',
+                                             'ipAddress': [new_ip]}}
+        fixed_vnic = {'addressGroups': {'addressGroups': [addr_group]},
+                      'type': 'uplink',
+                      'index': 1}
+
+        with mock.patch.object(self.nsxv_manager.vcns,
+            'get_interfaces', return_value=self.orig_vnics):
+            self.edge_manager.update_interface_addr(
+                self.ctx, self.edge_id, old_ip, new_ip,
+                self.subnet_mask, is_uplink=True)
+            self.nsxv_manager.vcns.update_interface.assert_called_once_with(
+                self.edge_id, fixed_vnic)
+
+    def test_add_secondary_interface_addr(self):
+        self._test_update_intereface_secondary_addr(
+            None, '192.168.111.31')
+
+    def test_update_interface_addr_fail(self):
+        # Old ip is not configured on the interface, so we should fail
+        old_ip = '192.168.111.32'
+        new_ip = '192.168.111.31'
+
+        with mock.patch.object(self.nsxv_manager.vcns,
+            'get_interfaces', return_value=self.orig_vnics):
+            self.assertRaises(
+                nsx_exc.NsxPluginException,
+                self.edge_manager.update_interface_addr,
+                self.ctx, self.edge_id, old_ip, new_ip,
+                self.subnet_mask, is_uplink=True)
+
+    def _test_update_vdr_intereface_primary_addr(self, old_ip,
+                                                 new_ip):
+        fixed_vnic = {'addressGroups':
+                      {'addressGroups': [
+                              {'subnetMask': self.subnet_mask,
+                               'subnetPrefixLength': self.pref_len,
+                               'primaryAddress': new_ip}] if new_ip else []},
+                      'type': 'internal',
+                      'index': 2}
+
+        with mock.patch.object(self.nsxv_manager.vcns,
+            'get_vdr_internal_interface', return_value=self.orig_vdr):
+            with mock.patch.object(self.nsxv_manager.vcns,
+                'update_vdr_internal_interface') as vcns_update:
+                self.edge_manager.update_vdr_interface_addr(
+                    self.ctx, self.edge_id, self.vnic, old_ip, new_ip,
+                    self.subnet_mask)
+                vcns_update.assert_called_once_with(self.edge_id,
+                                                    self.vnic,
+                                                    {'interface': fixed_vnic})
+
+    def test_update_vdr_interface_addr_intrernal(self):
+        self._test_update_vdr_intereface_primary_addr(
+            self.vdr_ip, '20.0.0.2')
+
+    def test_remove_vdr_interface_primary_addr_intrernal(self):
+        self._test_update_vdr_intereface_primary_addr(
+            self.vdr_ip, None)
+
+    def test_update_vdr_interface_addr_fail(self):
+        # Old ip is not configured on the vdr interface, so we should fail
+        old_ip = '192.168.111.32'
+        new_ip = '192.168.111.31'
+
+        with mock.patch.object(self.nsxv_manager.vcns,
+            'get_vdr_internal_interface', return_value=self.orig_vdr):
+            self.assertRaises(
+                nsx_exc.NsxPluginException,
+                self.edge_manager.update_vdr_interface_addr,
+                self.ctx, self.edge_id, self.vnic, old_ip, new_ip,
+                self.subnet_mask)
+
 
 class EdgeManagerTestCase(EdgeUtilsTestCaseMixin):
 
@@ -488,6 +660,7 @@ class EdgeManagerTestCase(EdgeUtilsTestCaseMixin):
                    nsxv_constants.SERVICE_EDGE + '-edge-' + str(0))
         self.nsxv_manager.update_edge.assert_has_calls(
             [mock.call('fake_id', edge_id, 'fake_name', None,
+                       jobdata=self.fake_jobdata, set_errors=True,
                        appliance_size=nsxv_constants.LARGE, dist=False)])
 
     def test_allocate_compact_edge_appliance_with_default(self):
@@ -505,6 +678,7 @@ class EdgeManagerTestCase(EdgeUtilsTestCaseMixin):
                    nsxv_constants.SERVICE_EDGE + '-edge-' + str(0))
         self.nsxv_manager.update_edge.assert_has_calls(
             [mock.call('fake_id', edge_id, 'fake_name', None,
+                       jobdata=self.fake_jobdata, set_errors=True,
                        appliance_size=nsxv_constants.COMPACT, dist=False)])
 
     def test_allocate_large_edge_appliance_with_vdr(self):
@@ -521,6 +695,7 @@ class EdgeManagerTestCase(EdgeUtilsTestCaseMixin):
                    nsxv_constants.VDR_EDGE + '-edge-' + str(0))
         self.nsxv_manager.update_edge.assert_has_calls(
             [mock.call('fake_id', edge_id, 'fake_name', None,
+                       jobdata=self.fake_jobdata, set_errors=True,
                        appliance_size=nsxv_constants.LARGE, dist=True)])
 
     def test_free_edge_appliance_with_empty(self):
